@@ -22,6 +22,10 @@
 #define CRLFILE "crl.pem"
 
 #define CHECK(x) assert((x)>=0)
+#define LOOP_CHECK(rval, cmd) \
+        do { \
+                rval = cmd; \
+        } while(rval == GNUTLS_E_AGAIN || rval == GNUTLS_E_INTERRUPTED)
 
 /* The OCSP status file contains up to date information about revocation
  * of the server's certificate. That can be periodically be updated
@@ -38,24 +42,6 @@
 #define MAX_BUF 1024
 #define PORT 5556               /* listen to 5556 port */
 
-/* These are global */
-static gnutls_dh_params_t dh_params;
-
-static int generate_dh_params(void)
-{
-        unsigned int bits = gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH,
-                                                        GNUTLS_SEC_PARAM_MEDIUM);
-
-        /* Generate Diffie-Hellman parameters - for use with DHE
-         * kx algorithms. When short bit length is used, it might
-         * be wise to regenerate parameters often.
-         */
-        CHECK(gnutls_dh_params_init(&dh_params));
-        CHECK(gnutls_dh_params_generate2(dh_params, bits));
-
-        return 0;
-}
-
 int main(void)
 {
         int listen_sd;
@@ -81,21 +67,35 @@ int main(void)
         CHECK(gnutls_certificate_set_x509_crl_file(x509_cred, CRLFILE,
                                                    GNUTLS_X509_FMT_PEM));
 
+        /* The following code sets the certificate key pair as well as, 
+         * an OCSP response which corresponds to it. It is possible
+         * to set multiple key-pairs and multiple OCSP status responses
+         * (the latter since 3.5.6). See the manual pages of the individual
+         * functions for more information.
+         */
         CHECK(gnutls_certificate_set_x509_key_file(x509_cred, CERTFILE,
                                                    KEYFILE,
                                                    GNUTLS_X509_FMT_PEM));
 
-        /* loads an OCSP status request if available */
         CHECK(gnutls_certificate_set_ocsp_status_request_file(x509_cred,
                                                               OCSP_STATUS_FILE,
                                                               0));
 
-        generate_dh_params();
+        CHECK(gnutls_priority_init(&priority_cache, NULL, NULL));
 
-        CHECK(gnutls_priority_init(&priority_cache,
-                                   "PERFORMANCE:%SERVER_PRECEDENCE", NULL));
+        /* Instead of the default options as shown above one could specify
+         * additional options such as server precedence in ciphersuite selection
+         * as follows:
+         * gnutls_priority_init2(&priority_cache,
+         *                       "%SERVER_PRECEDENCE",
+         *                       NULL, GNUTLS_PRIORITY_INIT_DEF_APPEND);
+	 */
 
-        gnutls_certificate_set_dh_params(x509_cred, dh_params);
+#if GNUTLS_VERSION_NUMBER >= 0x030506
+        /* only available since GnuTLS 3.5.6, on previous versions see
+         * gnutls_certificate_set_dh_params(). */
+        gnutls_certificate_set_known_dh_params(x509_cred, GNUTLS_SEC_PARAM_MEDIUM);
+#endif
 
         /* Socket operations
          */
@@ -129,6 +129,8 @@ int main(void)
                  */
                 gnutls_certificate_server_set_request(session,
                                                       GNUTLS_CERT_IGNORE);
+                gnutls_handshake_set_timeout(session,
+                                             GNUTLS_DEFAULT_HANDSHAKE_TIMEOUT);
 
                 sd = accept(listen_sd, (struct sockaddr *) &sa_cli,
                             &client_len);
@@ -139,11 +141,7 @@ int main(void)
 
                 gnutls_transport_set_int(session, sd);
 
-                do {
-                        ret = gnutls_handshake(session);
-                }
-                while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
-
+                LOOP_CHECK(ret, gnutls_handshake(session));
                 if (ret < 0) {
                         close(sd);
                         gnutls_deinit(session);
@@ -158,7 +156,7 @@ int main(void)
                 /* print_info(session); */
 
                 for (;;) {
-                        ret = gnutls_record_recv(session, buffer, MAX_BUF);
+                        LOOP_CHECK(ret, gnutls_record_recv(session, buffer, MAX_BUF));
 
                         if (ret == 0) {
                                 printf
@@ -182,7 +180,7 @@ int main(void)
                 printf("\n");
                 /* do not wait for the peer to close the connection.
                  */
-                CHECK(gnutls_bye(session, GNUTLS_SHUT_WR));
+                LOOP_CHECK(ret, gnutls_bye(session, GNUTLS_SHUT_WR));
 
                 close(sd);
                 gnutls_deinit(session);
