@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,6 +28,7 @@
 
 #ifdef ENABLE_CRYPTODEV
 
+#include <assert.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <crypto/cryptodev.h>
@@ -108,6 +109,9 @@ static int cryptodev_setiv(void *_ctx, const void *iv, size_t iv_size)
 {
 	struct cryptodev_ctx *ctx = _ctx;
 
+	if (iv_size > EALG_MAX_BLOCK_LEN)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	memcpy(ctx->iv, iv, iv_size);
 
 	return 0;
@@ -247,7 +251,7 @@ int _gnutls_cryptodev_init(void)
 			return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
 		}
 
-		/* Set close-on-exec (not really neede here) */
+		/* Set close-on-exec (not really needed here) */
 		if (fcntl(cfd, F_SETFD, 1) == -1) {
 			gnutls_assert();
 			return GNUTLS_E_CRYPTODEV_IOCTL_ERROR;
@@ -298,12 +302,15 @@ static const int gnutls_mac_map[] = {
 
 static int
 cryptodev_mac_fast(gnutls_mac_algorithm_t algo,
+		   const void *nonce, size_t nonce_size,
 		   const void *key, size_t key_size, const void *text,
 		   size_t text_size, void *digest)
 {
 	struct cryptodev_ctx ctx;
 	int ret;
 
+	assert(nonce_size == 0);
+
 	memset(&ctx, 0, sizeof(ctx));
 	ctx.cfd = _gnutls_cryptodev_fd;
 	ctx.sess.mac = gnutls_mac_map[algo];
