@@ -30,8 +30,8 @@
 #include "gnutls_int.h"
 #include "fips.h"
 #include <gnutls/crypto.h>
-#include "aes-x86.h"
-#include "x86-common.h"
+#include <aes-x86.h>
+#include <x86-common.h>
 
 struct x86_aes_xts_ctx {
 	AES_KEY block_key;
@@ -40,11 +40,12 @@ struct x86_aes_xts_ctx {
 	int enc;
 };
 
-static int x86_aes_xts_cipher_init(gnutls_cipher_algorithm_t algorithm,
-				   void **_ctx, int enc)
+static int
+x86_aes_xts_cipher_init(gnutls_cipher_algorithm_t algorithm, void **_ctx,
+					int enc)
 {
 	if (algorithm != GNUTLS_CIPHER_AES_128_XTS &&
-	    algorithm != GNUTLS_CIPHER_AES_256_XTS)
+		algorithm != GNUTLS_CIPHER_AES_256_XTS)
 		return GNUTLS_E_INVALID_REQUEST;
 
 	*_ctx = gnutls_calloc(1, sizeof(struct x86_aes_xts_ctx));
@@ -53,13 +54,13 @@ static int x86_aes_xts_cipher_init(gnutls_cipher_algorithm_t algorithm,
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	((struct x86_aes_xts_ctx *)(*_ctx))->enc = enc;
+	((struct x86_aes_xts_ctx *) (*_ctx))->enc = enc;
 
 	return 0;
 }
 
-static int x86_aes_xts_cipher_setkey(void *_ctx, const void *userkey,
-				     size_t keysize)
+static int
+x86_aes_xts_cipher_setkey(void *_ctx, const void *userkey, size_t keysize)
 {
 	struct x86_aes_xts_ctx *ctx = _ctx;
 	int ret;
@@ -70,8 +71,9 @@ static int x86_aes_xts_cipher_setkey(void *_ctx, const void *userkey,
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
 	/* Check key block according to FIPS-140-2 IG A.9 */
-	if (_gnutls_fips_mode_enabled()) {
+	if (_gnutls_fips_mode_enabled()){
 		if (gnutls_memcmp(key, key + (keysize / 2), keysize / 2) == 0) {
+			_gnutls_switch_lib_state(LIB_STATE_ERROR);
 			return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 		}
 	}
@@ -80,17 +82,20 @@ static int x86_aes_xts_cipher_setkey(void *_ctx, const void *userkey,
 	keybits = keysize * 4;
 
 	if (ctx->enc)
-		ret = aesni_set_encrypt_key(key, keybits,
-					    ALIGN16(&ctx->block_key));
+		ret =
+		    aesni_set_encrypt_key(key, keybits,
+					  ALIGN16(&ctx->block_key));
 	else
-		ret = aesni_set_decrypt_key(key, keybits,
-					    ALIGN16(&ctx->block_key));
+		ret =
+		    aesni_set_decrypt_key(key, keybits,
+					  ALIGN16(&ctx->block_key));
 
 	if (ret != 0)
 		return gnutls_assert_val(GNUTLS_E_ENCRYPTION_FAILED);
 
-	ret = aesni_set_encrypt_key(key + (keysize / 2), keybits,
-				    ALIGN16(&ctx->tweak_key));
+	ret =
+	    aesni_set_encrypt_key(key + (keysize / 2), keybits,
+					  ALIGN16(&ctx->tweak_key));
 	if (ret != 0)
 		return gnutls_assert_val(GNUTLS_E_ENCRYPTION_FAILED);
 
@@ -108,14 +113,12 @@ static int x86_aes_xts_setiv(void *_ctx, const void *iv, size_t iv_size)
 	return 0;
 }
 
-static int x86_aes_xts_encrypt(void *_ctx, const void *src, size_t src_size,
-			       void *dst, size_t dst_size)
+static int
+x86_aes_xts_encrypt(void *_ctx, const void *src, size_t src_size,
+	    void *dst, size_t dst_size)
 {
 	struct x86_aes_xts_ctx *ctx = _ctx;
 
-	if (unlikely(dst_size < src_size))
-		return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
-
 	if (src_size < 16)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
@@ -124,14 +127,12 @@ static int x86_aes_xts_encrypt(void *_ctx, const void *src, size_t src_size,
 	return 0;
 }
 
-static int x86_aes_xts_decrypt(void *_ctx, const void *src, size_t src_size,
-			       void *dst, size_t dst_size)
+static int
+x86_aes_xts_decrypt(void *_ctx, const void *src, size_t src_size,
+	    void *dst, size_t dst_size)
 {
 	struct x86_aes_xts_ctx *ctx = _ctx;
 
-	if (unlikely(dst_size < src_size))
-		return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
-
 	if (src_size < 16)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
@@ -144,7 +145,7 @@ static void x86_aes_xts_deinit(void *_ctx)
 {
 	struct x86_aes_xts_ctx *ctx = _ctx;
 
-	zeroize_key(ctx, sizeof(*ctx));
+	zeroize_temp_key(ctx, sizeof(*ctx));
 	gnutls_free(ctx);
 }
 
@@ -156,3 +157,4 @@ const gnutls_crypto_cipher_st _gnutls_aes_xts_x86_aesni = {
 	.decrypt = x86_aes_xts_decrypt,
 	.deinit = x86_aes_xts_deinit,
 };
+
