@@ -49,7 +49,7 @@ mac_entry_st hash_algorithms[] = {
 	 .id = GNUTLS_MAC_MD5_SHA1,
 	 .output_size = 36,
 	 .key_size = 36,
-	 .preimage_insecure = 1,
+	 .flags = GNUTLS_MAC_FLAG_PREIMAGE_INSECURE,
 	 .block_size = 64},
 	{.name = "SHA256",
 	 .oid = HASH_OID_SHA256,
@@ -121,25 +121,28 @@ mac_entry_st hash_algorithms[] = {
 	 .id = GNUTLS_MAC_MD5,
 	 .output_size = 16,
 	 .key_size = 16,
-	 .preimage_insecure = 1,
+	 .flags = GNUTLS_MAC_FLAG_PREIMAGE_INSECURE,
 	 .block_size = 64},
 	{.name = "MD2",
 	 .oid = HASH_OID_MD2,
-	 .preimage_insecure = 1,
+	 .flags = GNUTLS_MAC_FLAG_PREIMAGE_INSECURE,
 	 .id = GNUTLS_MAC_MD2},
 	{.name = "RIPEMD160",
 	 .oid = HASH_OID_RMD160,
 	 .id = GNUTLS_MAC_RMD160,
 	 .output_size = 20,
 	 .key_size = 20,
-	 .block_size = 64},
+	 .block_size = 64
+	},
 	{.name = "GOSTR341194",
 	 .oid = HASH_OID_GOST_R_3411_94,
 	 .mac_oid = MAC_OID_GOST_R_3411_94,
 	 .id = GNUTLS_MAC_GOSTR_94,
 	 .output_size = 32,
 	 .key_size = 32,
-	 .block_size = 32},
+	 .block_size = 32,
+	 .flags = GNUTLS_MAC_FLAG_PREIMAGE_INSECURE
+	},
 	{.name = "STREEBOG-256",
 	 .oid = HASH_OID_STREEBOG_256,
 	 .mac_oid = MAC_OID_STREEBOG_256,
@@ -177,6 +180,30 @@ mac_entry_st hash_algorithms[] = {
 	 .output_size = 16,
 	 .key_size = 32,
 	 .nonce_size = 12},
+	{.name = "GOST28147-TC26Z-IMIT",
+	 .id = GNUTLS_MAC_GOST28147_TC26Z_IMIT,
+	 .output_size = 4,
+	 .key_size = 32,
+	 .block_size = 8,
+	 .flags = GNUTLS_MAC_FLAG_CONTINUOUS_MAC},
+	{.name = "SHAKE-128",
+	 .oid = HASH_OID_SHAKE_128,
+	 .id = GNUTLS_MAC_SHAKE_128,
+	 .block_size = 168},
+	{.name = "SHAKE-256",
+	 .oid = HASH_OID_SHAKE_256,
+	 .id = GNUTLS_MAC_SHAKE_256,
+	 .block_size = 136},
+	{.name = "OMAC-MAGMA",
+	 .id = GNUTLS_MAC_MAGMA_OMAC,
+	 .output_size = 8,
+	 .key_size = 32,
+	 .block_size = 8},
+	{.name = "OMAC-KUZNYECHIK",
+	 .id = GNUTLS_MAC_KUZNYECHIK_OMAC,
+	 .output_size = 16,
+	 .key_size = 32,
+	 .block_size = 16},
 	{.name = "MAC-NULL",
 	 .id = GNUTLS_MAC_NULL},
 	{0, 0, 0, 0, 0, 0, 0, 0, 0}
@@ -271,7 +298,7 @@ int _gnutls_digest_mark_insecure(const char *name)
 
 	for(p = hash_algorithms; p->name != NULL; p++) {
 		if (p->oid != NULL && c_strcasecmp(p->name, name) == 0) {
-			p->preimage_insecure = 1;
+			p->flags |= GNUTLS_MAC_FLAG_PREIMAGE_INSECURE;
 			return 0;
 		}
 	}
@@ -286,7 +313,7 @@ unsigned _gnutls_digest_is_insecure(gnutls_digest_algorithm_t dig)
 
 	for(p = hash_algorithms; p->name != NULL; p++) {
 		if (p->oid != NULL && p->id == (gnutls_mac_algorithm_t)dig) {
-			return p->preimage_insecure;
+			return p->flags & GNUTLS_MAC_FLAG_PREIMAGE_INSECURE;
 		}
 	}
 
