@@ -543,7 +543,7 @@ read_cert_file(gnutls_certificate_credentials_t res,
 		return read_cert_url(res, key, certfile);
 	}
 
-	data = read_binary_file(certfile, &size);
+	data = read_file(certfile, RF_BINARY, &size);
 
 	if (data == NULL) {
 		gnutls_assert();
@@ -588,7 +588,7 @@ _gnutls_read_key_file(gnutls_certificate_credentials_t res,
 			    (GNUTLS_E_UNIMPLEMENTED_FEATURE);
 	}
 
-	data = read_binary_file(keyfile, &size);
+	data = read_file(keyfile, RF_BINARY | RF_SENSITIVE, &size);
 
 	if (data == NULL) {
 		gnutls_assert();
@@ -596,6 +596,7 @@ _gnutls_read_key_file(gnutls_certificate_credentials_t res,
 	}
 
 	ret = _gnutls_read_key_mem(res, data, size, type, pass, flags, rkey);
+	zeroize_key(data, size);
 	free(data);
 
 	return ret;
@@ -738,10 +739,14 @@ gnutls_certificate_set_x509_key(gnutls_certificate_credentials_t res,
 				gnutls_x509_privkey_t key)
 {
 	int ret;
+	int npcerts = 0;
 	gnutls_privkey_t pkey;
 	gnutls_pcert_st *pcerts = NULL;
 	gnutls_str_array_t names;
 
+	if (cert_list == NULL || cert_list_size < 1)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	_gnutls_str_array_init(&names);
 
 	/* this should be first
@@ -784,10 +789,11 @@ gnutls_certificate_set_x509_key(gnutls_certificate_credentials_t res,
 		gnutls_assert();
 		goto cleanup;
 	}
+	npcerts = cert_list_size;
 
 	ret =
 	    _gnutls_certificate_credential_append_keypair(res, pkey, names, pcerts,
-						   cert_list_size);
+						   npcerts);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
@@ -806,6 +812,8 @@ gnutls_certificate_set_x509_key(gnutls_certificate_credentials_t res,
 	CRED_RET_SUCCESS(res);
 
       cleanup:
+	while (npcerts-- > 0)
+		gnutls_pcert_deinit(&pcerts[npcerts]);
 	gnutls_free(pcerts);
 	_gnutls_str_array_clear(&names);
 	return ret;
@@ -1138,8 +1146,12 @@ gnutls_certificate_set_x509_trust(gnutls_certificate_credentials_t res,
 				  int ca_list_size)
 {
 	int ret, i, j;
-	gnutls_x509_crt_t *new_list = gnutls_malloc(ca_list_size * sizeof(gnutls_x509_crt_t));
+	gnutls_x509_crt_t *new_list;
 
+	if (ca_list == NULL || ca_list_size < 1)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	new_list = gnutls_malloc(ca_list_size * sizeof(gnutls_x509_crt_t));
 	if (!new_list)
 		return GNUTLS_E_MEMORY_ERROR;
 
@@ -1447,7 +1459,8 @@ int
 	size_t size;
 	int ret;
 
-	p12blob.data = (void *) read_binary_file(pkcs12file, &size);
+	p12blob.data = (void *) read_file(pkcs12file, RF_BINARY | RF_SENSITIVE,
+					  &size);
 	p12blob.size = (unsigned int) size;
 	if (p12blob.data == NULL) {
 		gnutls_assert();
@@ -1457,7 +1470,9 @@ int
 	ret =
 	    gnutls_certificate_set_x509_simple_pkcs12_mem(res, &p12blob,
 							  type, password);
+	zeroize_key(p12blob.data, p12blob.size);
 	free(p12blob.data);
+	p12blob.size = 0;
 
 	return ret;
 }
