@@ -20,8 +20,8 @@
  *
  */
 
-#ifndef GNUTLS_CIPHER_INT
-#define GNUTLS_CIPHER_INT
+#ifndef GNUTLS_LIB_CIPHER_INT_H
+#define GNUTLS_LIB_CIPHER_INT_H
 
 #include <gnutls/crypto.h>
 #include "errors.h"
@@ -50,6 +50,9 @@ typedef void (*cipher_deinit_func) (void *hd);
 
 typedef int (*cipher_auth_func) (void *hd, const void *data, size_t);
 typedef int (*cipher_setiv_func) (void *hd, const void *iv, size_t);
+typedef int (*cipher_getiv_func) (void *hd, void *iv, size_t);
+
+typedef int (*cipher_setkey_func) (void *hd, const void *key, size_t keysize);
 
 typedef void (*cipher_tag_func) (void *hd, void *tag, size_t);
 
@@ -63,6 +66,8 @@ typedef struct {
 	cipher_auth_func auth;
 	cipher_tag_func tag;
 	cipher_setiv_func setiv;
+	cipher_getiv_func getiv;
+	cipher_setkey_func setkey;
 	cipher_deinit_func deinit;
 } cipher_hd_st;
 
@@ -76,6 +81,22 @@ inline static int _gnutls_cipher_setiv(const cipher_hd_st * handle,
 	return handle->setiv(handle->handle, iv, ivlen);
 }
 
+inline static int _gnutls_cipher_getiv(const cipher_hd_st * handle,
+				       void *iv, size_t ivlen)
+{
+	if (unlikely(handle == NULL || handle->handle == NULL ||
+		    handle->getiv == NULL))
+		return GNUTLS_E_INVALID_REQUEST;
+
+	return handle->getiv(handle->handle, iv, ivlen);
+}
+
+inline static int _gnutls_cipher_setkey(const cipher_hd_st * handle,
+					const void *key, size_t keylen)
+{
+	return handle->setkey(handle->handle, key, keylen);
+}
+
 inline static int
 _gnutls_cipher_encrypt2(const cipher_hd_st * handle, const void *text,
 			size_t textlen, void *ciphertext,
@@ -158,6 +179,11 @@ inline static void _gnutls_cipher_deinit(cipher_hd_st * handle)
 
 int _gnutls_cipher_exists(gnutls_cipher_algorithm_t cipher);
 
+int _gnutls_cipher_get_iv(gnutls_cipher_hd_t handle, void *iv,
+			  size_t ivlen);
+
+int _gnutls_cipher_set_key(gnutls_cipher_hd_t handle, void *key, size_t keylen);
+
 #define _gnutls_cipher_is_aead(h) _gnutls_cipher_algo_is_aead((h)->e)
 
 /* returns the tag in AUTHENC ciphers */
@@ -195,6 +221,9 @@ typedef struct {
 	unsigned int is_mac:1;
 #ifdef ENABLE_SSL3
 	unsigned int ssl_hmac:1;
+#endif
+#ifdef ENABLE_GOST
+	unsigned int continuous_mac:1;
 #endif
 	unsigned int non_null:1;
 	unsigned int etm:1;
@@ -226,11 +255,11 @@ int _gnutls_auth_cipher_decrypt2(auth_cipher_hd_st * handle,
 int _gnutls_auth_cipher_tag(auth_cipher_hd_st * handle, void *tag,
 			    int tag_size);
 
-inline static void _gnutls_auth_cipher_setiv(const auth_cipher_hd_st *
+inline static int _gnutls_auth_cipher_setiv(const auth_cipher_hd_st *
 					     handle, const void *iv,
 					     size_t ivlen)
 {
-	_gnutls_cipher_setiv(&handle->cipher, iv, ivlen);
+	return _gnutls_cipher_setiv(&handle->cipher, iv, ivlen);
 }
 
 inline static size_t _gnutls_auth_cipher_tag_len(auth_cipher_hd_st *
@@ -244,4 +273,4 @@ inline static size_t _gnutls_auth_cipher_tag_len(auth_cipher_hd_st *
 void _gnutls_auth_cipher_deinit(auth_cipher_hd_st * handle);
 
 
-#endif				/* GNUTLS_CIPHER_INT */
+#endif /* GNUTLS_LIB_CIPHER_INT_H */
