@@ -20,11 +20,11 @@
  *
  */
 
-#ifndef EXT_SUPPORTED_VERSIONS_H
-#define EXT_SUPPORTED_VERSIONS_H
+#ifndef GNUTLS_LIB_EXT_SUPPORTED_VERSIONS_H
+#define GNUTLS_LIB_EXT_SUPPORTED_VERSIONS_H
 
 #include <hello_ext.h>
 
 extern const hello_ext_entry_st ext_mod_supported_versions;
 
-#endif
+#endif /* GNUTLS_LIB_EXT_SUPPORTED_VERSIONS_H */
