@@ -53,8 +53,6 @@ typedef int ssize_t;
 #endif
 #include <time.h>
 
-#include <nettle/memxor.h>
-
 #include "attribute.h"
 
 #define ENABLE_ALIGN16
@@ -361,9 +359,6 @@ verify(GNUTLS_EXTENSION_MAX_VALUE - GNUTLS_EXTENSION_MAX >= 16);
 
 typedef enum { CIPHER_STREAM, CIPHER_BLOCK, CIPHER_AEAD } cipher_type_t;
 
-#define RESUME_TRUE 1
-#define RESUME_FALSE 0
-
 /* Record Protocol */
 typedef enum content_type_t {
 	GNUTLS_CHANGE_CIPHER_SPEC = 20, GNUTLS_ALERT,
@@ -851,6 +846,7 @@ struct record_state_st {
 	unsigned aead_tag_size;
 	unsigned is_aead;
 	uint64_t sequence_number;
+	gnutls_record_encryption_level_t level;
 };
 
 
@@ -1085,7 +1081,7 @@ typedef struct {
 	gnutls_buffer_st handshake_hash_buffer;	/* used to keep the last received handshake
 						 * message */
 
-	bool resumable;	/* TRUE or FALSE - if we can resume that session */
+	bool resumable;	/* if we can resume that session */
 
 	send_ticket_state_t ticket_state; /* used by gnutls_session_ticket_send() */
 	bye_state_t bye_state; /* used by gnutls_bye() */
@@ -1099,7 +1095,7 @@ typedef struct {
 						 * no interruption has happened.
 						 */
 
-	bool invalid_connection;	/* true or FALSE - if this session is valid */
+	bool invalid_connection;	/* if this session is valid */
 
 	bool may_not_read;	/* if it's 0 then we can read/write, otherwise it's forbidden to read/write
 				 */
@@ -1134,7 +1130,7 @@ typedef struct {
 	uint16_t dh_prime_bits; /* srp_prime_bits */
 
 	/* resumed session */
-	bool resumed;	/* RESUME_TRUE or FALSE - if we are resuming a session */
+	bool resumed;	/* if we are resuming a session */
 
 	/* server side: non-zero if resumption was requested by client
 	 * client side: non-zero if we set resumption parameters */
@@ -1196,6 +1192,11 @@ typedef struct {
 	 */
 	gnutls_certificate_request_t send_cert_req;
 
+	/* callback to print the full path of certificate
+	 * validation to the trusted root.
+	 */
+	gnutls_verify_output_function *cert_output_callback;
+
 	size_t max_handshake_data_buffer_size;
 
 	/* PUSH & PULL functions.
@@ -1226,6 +1227,9 @@ typedef struct {
 	gnutls_handshake_hook_func h_hook;
 	unsigned int h_type;	/* the hooked type */
 	int16_t h_post;		/* whether post-generation/receive */
+	gnutls_handshake_read_func h_read_func;
+	gnutls_handshake_secret_func h_secret_func;
+	gnutls_alert_read_func alert_read_func;
 
 	gnutls_keylog_func keylog_func;
 
