@@ -210,7 +210,7 @@ int _gnutls13_handshake_client(gnutls_session_t session)
 
 	SAVE_TRANSCRIPT;
 
-	if (session->internals.resumed != RESUME_FALSE)
+	if (session->internals.resumed)
 		_gnutls_set_resumed_parameters(session);
 
 	return 0;
@@ -325,7 +325,7 @@ static int generate_hs_traffic_keys(gnutls_session_t session)
 	if ((session->security_parameters.entity == GNUTLS_CLIENT &&
 	      (!(session->internals.hsk_flags & HSK_KEY_SHARE_RECEIVED) ||
 	        (!(session->internals.hsk_flags & HSK_PSK_KE_MODE_DHE_PSK) &&
-	           session->internals.resumed != RESUME_FALSE))) ||
+	           session->internals.resumed))) ||
 	    (session->security_parameters.entity == GNUTLS_SERVER &&
 	      !(session->internals.hsk_flags & HSK_KEY_SHARE_SENT))) {
 
@@ -506,12 +506,13 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 
 		FALLTHROUGH;
 	case STATE109:
-		if (session->internals.resumed != RESUME_FALSE)
+		if (session->internals.resumed)
 			_gnutls_set_resumed_parameters(session);
 
 		if (session->internals.hsk_flags & HSK_EARLY_START_USED) {
-			ret = _gnutls13_send_session_ticket(session, TLS13_TICKETS_TO_SEND,
-							    AGAIN(STATE109));
+			if (!(session->internals.flags & GNUTLS_NO_AUTO_SEND_TICKET))
+				ret = _gnutls13_send_session_ticket(session, TLS13_TICKETS_TO_SEND,
+								    AGAIN(STATE109));
 
 			STATE = STATE109;
 			IMED_RET("send session ticket", ret, 0);
@@ -565,7 +566,8 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 
 		FALLTHROUGH;
 	case STATE115:
-		if (!(session->internals.hsk_flags & (HSK_TLS13_TICKET_SENT|HSK_EARLY_START_USED))) {
+		if (!(session->internals.hsk_flags & (HSK_TLS13_TICKET_SENT|HSK_EARLY_START_USED)) &&
+		    !(session->internals.flags & GNUTLS_NO_AUTO_SEND_TICKET)) {
 			ret = _gnutls13_send_session_ticket(session, TLS13_TICKETS_TO_SEND,
 							    AGAIN(STATE115));
 			STATE = STATE115;
