@@ -46,11 +46,12 @@
  *    of record boundaries.
  */
 
+
 /* Initialize a buffer head.
  *
  * Cost: O(1)
  */
-void _mbuffer_head_init(mbuffer_head_st *buf)
+void _mbuffer_head_init(mbuffer_head_st * buf)
 {
 	buf->head = NULL;
 	buf->tail = NULL;
@@ -64,7 +65,7 @@ void _mbuffer_head_init(mbuffer_head_st *buf)
  * Cost: O(n)
  * n: Number of segments currently in the buffer.
  */
-void _mbuffer_head_clear(mbuffer_head_st *buf)
+void _mbuffer_head_clear(mbuffer_head_st * buf)
 {
 	mbuffer_st *bufel, *next;
 
@@ -80,7 +81,7 @@ void _mbuffer_head_clear(mbuffer_head_st *buf)
  *
  * Cost: O(1)
  */
-void _mbuffer_enqueue(mbuffer_head_st *buf, mbuffer_st *bufel)
+void _mbuffer_enqueue(mbuffer_head_st * buf, mbuffer_st * bufel)
 {
 	bufel->next = NULL;
 	bufel->prev = buf->tail;
@@ -101,14 +102,14 @@ void _mbuffer_enqueue(mbuffer_head_st *buf, mbuffer_st *bufel)
  *
  * Returns the buffer following it.
  */
-mbuffer_st *_mbuffer_dequeue(mbuffer_head_st *buf, mbuffer_st *bufel)
+mbuffer_st *_mbuffer_dequeue(mbuffer_head_st * buf, mbuffer_st * bufel)
 {
 	mbuffer_st *ret = bufel->next;
 
-	if (buf->tail == bufel) /* if last */
+	if (buf->tail == bufel)	/* if last */
 		buf->tail = bufel->prev;
 
-	if (buf->head == bufel) /* if first */
+	if (buf->head == bufel)	/* if first */
 		buf->head = bufel->next;
 
 	if (bufel->prev)
@@ -129,7 +130,7 @@ mbuffer_st *_mbuffer_dequeue(mbuffer_head_st *buf, mbuffer_st *bufel)
  *
  * Cost: O(1)
  */
-void _mbuffer_head_push_first(mbuffer_head_st *buf, mbuffer_st *bufel)
+void _mbuffer_head_push_first(mbuffer_head_st * buf, mbuffer_st * bufel)
 {
 	bufel->prev = NULL;
 	bufel->next = buf->head;
@@ -151,7 +152,7 @@ void _mbuffer_head_push_first(mbuffer_head_st *buf, mbuffer_st *bufel)
  *
  * Cost: O(1)
  */
-mbuffer_st *_mbuffer_head_pop_first(mbuffer_head_st *buf)
+mbuffer_st *_mbuffer_head_pop_first(mbuffer_head_st * buf)
 {
 	mbuffer_st *bufel = buf->head;
 
@@ -169,7 +170,8 @@ mbuffer_st *_mbuffer_head_pop_first(mbuffer_head_st *buf)
  *
  * Cost: O(1)
  */
-mbuffer_st *_mbuffer_head_get_first(mbuffer_head_st *buf, gnutls_datum_t *msg)
+mbuffer_st *_mbuffer_head_get_first(mbuffer_head_st * buf,
+				    gnutls_datum_t * msg)
 {
 	mbuffer_st *bufel = buf->head;
 
@@ -191,7 +193,7 @@ mbuffer_st *_mbuffer_head_get_first(mbuffer_head_st *buf, gnutls_datum_t *msg)
  *
  * Cost: O(1)
  */
-mbuffer_st *_mbuffer_head_get_next(mbuffer_st *cur, gnutls_datum_t *msg)
+mbuffer_st *_mbuffer_head_get_next(mbuffer_st * cur, gnutls_datum_t * msg)
 {
 	mbuffer_st *bufel = cur->next;
 
@@ -214,7 +216,7 @@ mbuffer_st *_mbuffer_head_get_next(mbuffer_st *cur, gnutls_datum_t *msg)
  *
  * Cost: O(1)
  */
-static inline void remove_front(mbuffer_head_st *buf)
+static inline void remove_front(mbuffer_head_st * buf)
 {
 	mbuffer_st *bufel = buf->head;
 
@@ -235,7 +237,7 @@ static inline void remove_front(mbuffer_head_st *buf)
  * Cost: O(n)
  * n: Number of segments needed to remove the specified amount of data.
  */
-int _mbuffer_head_remove_bytes(mbuffer_head_st *buf, size_t bytes)
+int _mbuffer_head_remove_bytes(mbuffer_head_st * buf, size_t bytes)
 {
 	size_t left = bytes;
 	mbuffer_st *bufel, *next;
@@ -285,13 +287,14 @@ mbuffer_st *_mbuffer_alloc(size_t maximum_size)
 	memset(st, 0, sizeof(*st));
 
 	/* payload points after the mbuffer_st structure */
-	st->msg.data = (uint8_t *)st + sizeof(mbuffer_st);
+	st->msg.data = (uint8_t *) st + sizeof(mbuffer_st);
 	st->msg.size = 0;
 	st->maximum_size = maximum_size;
 
 	return st;
 }
 
+
 /* Copy data into a segment. The segment must not be part of a buffer
  * head when using this function.
  *
@@ -302,7 +305,9 @@ mbuffer_st *_mbuffer_alloc(size_t maximum_size)
  * Cost: O(n)
  * n: number of bytes to copy
  */
-int _mbuffer_append_data(mbuffer_st *bufel, void *newdata, size_t newdata_size)
+int
+_mbuffer_append_data(mbuffer_st * bufel, void *newdata,
+		     size_t newdata_size)
 {
 	if (bufel->msg.size + newdata_size <= bufel->maximum_size) {
 		memcpy(&bufel->msg.data[bufel->msg.size], newdata,
@@ -317,7 +322,7 @@ int _mbuffer_append_data(mbuffer_st *bufel, void *newdata, size_t newdata_size)
 }
 
 #ifdef ENABLE_ALIGN16
-#define ALIGN_SIZE 16
+# define ALIGN_SIZE 16
 
 /* Allocate a 16-byte aligned buffer segment. The segment is not initially "owned" by
  * any buffer.
@@ -347,9 +352,9 @@ mbuffer_st *_mbuffer_alloc_align16(size_t maximum_size, unsigned align_pos)
 	memset(st, 0, sizeof(*st));
 
 	/* payload points after the mbuffer_st structure */
-	st->msg.data = (uint8_t *)st + sizeof(mbuffer_st);
-
-	cur_alignment = ((size_t)(st->msg.data + align_pos)) % ALIGN_SIZE;
+	st->msg.data = (uint8_t *) st + sizeof(mbuffer_st);
+	
+	cur_alignment = ((size_t)(st->msg.data+align_pos)) % ALIGN_SIZE;
 	if (cur_alignment > 0)
 		st->msg.data += ALIGN_SIZE - cur_alignment;
 
@@ -359,11 +364,11 @@ mbuffer_st *_mbuffer_alloc_align16(size_t maximum_size, unsigned align_pos)
 	return st;
 }
 
-static unsigned is_aligned16(mbuffer_st *bufel, unsigned align_pos)
+static unsigned is_aligned16(mbuffer_st * bufel, unsigned align_pos)
 {
-	uint8_t *ptr = _mbuffer_get_udata_ptr(bufel);
+	uint8_t * ptr = _mbuffer_get_udata_ptr(bufel);
 
-	if (((size_t)(ptr + align_pos)) % ALIGN_SIZE == 0)
+	if (((size_t)(ptr+align_pos)) % ALIGN_SIZE == 0)
 		return 1;
 	else
 		return 0;
@@ -377,7 +382,7 @@ static unsigned is_aligned16(mbuffer_st *bufel, unsigned align_pos)
  * Cost: O(n)
  * n: number of segments initially in the buffer
  */
-int _mbuffer_linearize_align16(mbuffer_head_st *buf, unsigned align_pos)
+int _mbuffer_linearize_align16(mbuffer_head_st * buf, unsigned align_pos)
 {
 	mbuffer_st *bufel, *cur;
 	gnutls_datum_t msg;
@@ -387,7 +392,7 @@ int _mbuffer_linearize_align16(mbuffer_head_st *buf, unsigned align_pos)
 		/* Nothing to do */
 		return 0;
 	}
-
+	
 	bufel = _mbuffer_head_get_first(buf, NULL);
 	if (buf->length == 1 && is_aligned16(bufel, align_pos))
 		return 0;
@@ -398,10 +403,8 @@ int _mbuffer_linearize_align16(mbuffer_head_st *buf, unsigned align_pos)
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	bufel->type = _mbuffer_head_get_first(buf, NULL)->type;
-
-	for (cur = _mbuffer_head_get_first(buf, &msg); msg.data != NULL;
-	     cur = _mbuffer_head_get_next(cur, &msg)) {
+	for (cur = _mbuffer_head_get_first(buf, &msg);
+	     msg.data != NULL; cur = _mbuffer_head_get_next(cur, &msg)) {
 		memcpy(&bufel->msg.data[pos], msg.data, msg.size);
 		bufel->msg.size += msg.size;
 		pos += msg.size;
@@ -413,7 +416,7 @@ int _mbuffer_linearize_align16(mbuffer_head_st *buf, unsigned align_pos)
 	return 0;
 }
 #else
-int _mbuffer_linearize(mbuffer_head_st *buf)
+int _mbuffer_linearize(mbuffer_head_st * buf)
 {
 	mbuffer_st *bufel, *cur;
 	gnutls_datum_t msg;
@@ -423,17 +426,15 @@ int _mbuffer_linearize(mbuffer_head_st *buf)
 		/* Nothing to do */
 		return 0;
 	}
-
+	
 	bufel = _mbuffer_alloc(buf->byte_length);
 	if (!bufel) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	bufel->type = _mbuffer_head_get_first(buf, NULL)->type;
-
-	for (cur = _mbuffer_head_get_first(buf, &msg); msg.data != NULL;
-	     cur = _mbuffer_head_get_next(cur, &msg)) {
+	for (cur = _mbuffer_head_get_first(buf, &msg);
+	     msg.data != NULL; cur = _mbuffer_head_get_next(cur, &msg)) {
 		memcpy(&bufel->msg.data[pos], msg.data, msg.size);
 		bufel->msg.size += msg.size;
 		pos += msg.size;
