@@ -22,10 +22,10 @@
  */
 
 #ifndef GNUTLS_NAME_VAL_ARRAY_H
-# define GNUTLS_NAME_VAL_ARRAY_H
+#define GNUTLS_NAME_VAL_ARRAY_H
 
-# include "gnutls_int.h"
-# include "errors.h"
+#include "gnutls_int.h"
+#include "errors.h"
 
 /* Functionality to allow an array of strings. Strings
  * are allowed to be added to the list and matched against it.
@@ -38,6 +38,11 @@ typedef struct name_val_array_st {
 	struct name_val_array_st *next;
 } *name_val_array_t;
 
+inline static void _name_val_array_init(name_val_array_t * head)
+{
+	*head = NULL;
+}
+
 inline static void _name_val_array_clear(name_val_array_t * head)
 {
 	name_val_array_t prev, array = *head;
@@ -51,8 +56,7 @@ inline static void _name_val_array_clear(name_val_array_t * head)
 }
 
 inline static const char *_name_val_array_value(name_val_array_t head,
-						const char *name,
-						unsigned name_size)
+					        const char *name, unsigned name_size)
 {
 	name_val_array_t array = head;
 
@@ -68,15 +72,15 @@ inline static const char *_name_val_array_value(name_val_array_t head,
 }
 
 inline static void append(name_val_array_t array, const char *name,
-			  unsigned name_len, const char *val, unsigned val_len)
+			  unsigned name_len, const char *val,
+			  unsigned val_len)
 {
-	array->name = ((char *)array) + sizeof(struct name_val_array_st);
+	array->name = ((char *) array) + sizeof(struct name_val_array_st);
 	memcpy(array->name, name, name_len);
 	array->name[name_len] = 0;
 	array->name_size = name_len;
 
-	array->val =
-	    ((char *)array) + name_len + 1 + sizeof(struct name_val_array_st);
+	array->val = ((char *) array) + name_len + 1 + sizeof(struct name_val_array_st);
 	if (val)
 		memcpy(array->val, val, val_len);
 	array->val[val_len] = 0;
@@ -85,11 +89,12 @@ inline static void append(name_val_array_t array, const char *name,
 }
 
 inline static int _name_val_array_append(name_val_array_t * head,
-					 const char *name, const char *val)
+					 const char *name,
+					 const char *val)
 {
 	name_val_array_t prev, array;
 	unsigned name_len = strlen(name);
-	unsigned val_len = (val == NULL) ? 0 : strlen(val);
+	unsigned val_len = (val==NULL)?0:strlen(val);
 
 	if (*head == NULL) {
 		*head =
