@@ -31,8 +31,10 @@
    not, see https://www.gnu.org/licenses/.
 */
  
-#ifndef NETTLE_STREEBOG_H_INCLUDED
-#define NETTLE_STREEBOG_H_INCLUDED
+#ifndef GNUTLS_LIB_NETTLE_GOST_STREEBOG_H
+#define GNUTLS_LIB_NETTLE_GOST_STREEBOG_H
+
+#include "config.h"
 
 #include <nettle/nettle-types.h>
 #include <nettle/nettle-meta.h>
@@ -41,6 +43,7 @@
 extern "C" {
 #endif
 
+#ifndef HAVE_NETTLE_STREEBOG512_UPDATE
 /* Name mangling */
 #define streebog256_init _gnutls_streebog256_init
 #define streebog256_digest _gnutls_streebog256_digest
@@ -97,9 +100,10 @@ streebog256_digest(struct streebog256_ctx *ctx,
 #define nettle_streebog512 _gnutls_streebog512
 extern const struct nettle_hash _gnutls_streebog256;
 extern const struct nettle_hash _gnutls_streebog512;
+#endif
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* NETTLE_STREEBOG_H_INCLUDED */
+#endif /* GNUTLS_LIB_NETTLE_GOST_STREEBOG_H */
