@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -26,16 +26,25 @@
 #include "errors.h"
 
 #include <sys/socket.h>
+#include <netinet/in.h> /* IPPROTO_TCP */
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <c-ctype.h>
 
 /* Get TCP_FASTOPEN */
 #ifdef HAVE_NETINET_TCP_H
 # include <netinet/tcp.h>
 #endif
 
+/* TCP Fast Open on OSX behaves differently from Linux, so define these helpers */
+#if defined __APPLE__ && defined __MACH__ && defined CONNECT_DATA_IDEMPOTENT && defined CONNECT_RESUME_ON_READ_WRITE
+# if defined __has_builtin && __has_builtin(__builtin_available)
+#  define TCP_FASTOPEN_OSX
+# endif
+#elif defined TCP_FASTOPEN && defined MSG_FASTOPEN
+# define TCP_FASTOPEN_LINUX
+#endif
+
 /* Do not use the gnulib functions for sending and receiving data.
  * Using them makes gnutls only working with gnulib applications.
  */
@@ -46,7 +55,7 @@
 
 #ifdef _WIN32
 static ssize_t
-tfo_send(gnutls_transport_ptr_t ptr, const void *buf, size_t len)
+tfo_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec, int iovec_cnt)
 {
 	tfo_st *p = ptr;
 	int fd = p->fd;
@@ -54,7 +63,8 @@ tfo_send(gnutls_transport_ptr_t ptr, const void *buf, size_t len)
 	if (unlikely(p->connect_addrlen != 0)) {
 		int ret;
 
-		ret = connect(fd, (struct sockaddr*)&p->connect_addr, p->connect_addrlen);
+		ret = connect(fd, (struct sockaddr*)&p->connect_addr,
+				p->connect_addrlen);
 		if (ret == -1 && (errno == EINPROGRESS)) {
 			gnutls_assert();
 			errno = EAGAIN;
@@ -68,7 +78,7 @@ tfo_send(gnutls_transport_ptr_t ptr, const void *buf, size_t len)
 		return ret;
 	}
 
-	return send(fd, buf, len, 0);
+	return system_writev(GNUTLS_INT_TO_POINTER(fd), iovec, iovec_cnt);
 }
 #else /* sendmsg */
 static ssize_t
@@ -77,7 +87,7 @@ tfo_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec, int iovec_cnt)
 	tfo_st *p = ptr;
 	int fd = p->fd;
 	struct msghdr hdr;
-	int ret, on = 1;
+	int ret;
 
 	memset(&hdr, 0, sizeof(hdr));
 	hdr.msg_iov = (struct iovec *)iovec;
@@ -86,8 +96,10 @@ tfo_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec, int iovec_cnt)
 	if (likely(!p->connect_addrlen))
 		return sendmsg(fd, &hdr, p->flags);
 
-#ifdef MSG_FASTOPEN
+# if defined(TCP_FASTOPEN_LINUX)
 	if (!p->connect_only) {
+		int on = 1;
+
 		if (setsockopt(fd, IPPROTO_TCP, TCP_FASTOPEN, &on, sizeof(on)) == -1)
 			_gnutls_debug_log("Failed to set socket option FASTOPEN\n");
 
@@ -107,9 +119,7 @@ tfo_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec, int iovec_cnt)
 				goto connect_only;
 			}
 		}
-	} else
-#endif
-	{
+	} else {
  connect_only:
 		ret = connect(fd, (struct sockaddr*)&p->connect_addr, p->connect_addrlen);
 		if (errno == ENOTCONN || errno == EINPROGRESS) {
@@ -120,6 +130,29 @@ tfo_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec, int iovec_cnt)
 		if (ret == 0)
 			p->connect_only = 0;
 	}
+# elif defined(TCP_FASTOPEN_OSX)
+	{
+		if(__builtin_available(macOS 10.11, iOS 9.0, tvOS 9.0, watchOS 2.0, *)) {
+			sa_endpoints_t endpoints = { .sae_dstaddr = (struct sockaddr*)&p->connect_addr, .sae_dstaddrlen = p->connect_addrlen };
+
+			ret = connectx(fd, &endpoints, SAE_ASSOCID_ANY, CONNECT_RESUME_ON_READ_WRITE | CONNECT_DATA_IDEMPOTENT, NULL, 0, NULL, NULL);
+		}
+		else
+		{
+			ret = connect(fd, (struct sockaddr*)&p->connect_addr, p->connect_addrlen);
+		}
+		if (errno == ENOTCONN || errno == EINPROGRESS) {
+			gnutls_assert();
+			errno = EAGAIN;
+		}
+	}
+# else
+	ret = connect(fd, (struct sockaddr*)&p->connect_addr, p->connect_addrlen);
+	if (errno == ENOTCONN || errno == EINPROGRESS) {
+		gnutls_assert();
+		errno = EAGAIN;
+	}
+# endif
 
 	if (ret == 0 || errno != EAGAIN) {
 		/* This has to be called just once, connect info not needed any more */
@@ -128,7 +161,7 @@ tfo_writev(gnutls_transport_ptr_t ptr, const giovec_t * iovec, int iovec_cnt)
 
 	return ret;
 }
-#endif
+#endif /* sendmsg */
 
 static
 int tfo_recv_timeout(gnutls_transport_ptr_t ptr, unsigned int ms)
@@ -185,7 +218,7 @@ gnutls_transport_set_fastopen(gnutls_session_t session,
 {
 	if (connect_addrlen > (socklen_t)sizeof(session->internals.tfo.connect_addr)) {
 		gnutls_assert();
-		abort();
+		return;
 	}
 
 	if (session->security_parameters.entity == GNUTLS_SERVER) {
@@ -207,11 +240,6 @@ gnutls_transport_set_fastopen(gnutls_session_t session,
 		session->internals.tfo.flags |= MSG_NOSIGNAL;
 #endif
 
-#ifdef _WIN32
-	gnutls_transport_set_vec_push_function(session, NULL);
-	gnutls_transport_set_push_function(session, tfo_send);
-#else
 	gnutls_transport_set_vec_push_function(session, tfo_writev);
-#endif
 }
 
