@@ -1,5 +1,5 @@
 /* Conversion UCS-4 to UTF-16.
-   Copyright (C) 2002, 2006-2007, 2009-2019 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2006-2007, 2009-2020 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2002.
 
    This program is free software: you can redistribute it and/or
@@ -30,7 +30,7 @@
 #include "unistr.h"
 
 int
-u16_uctomb_aux (uint16_t *s, ucs4_t uc, int n)
+u16_uctomb_aux (uint16_t *s, ucs4_t uc, ptrdiff_t n)
 {
   if (uc < 0xd800)
     {
