@@ -39,19 +39,9 @@ static int
 data2hex(const void *data, size_t data_size,
 	 gnutls_datum_t *out);
 
-struct oid_to_string {
-	const char *oid;
-	unsigned oid_size;
-	const char *ldap_desc;
-	unsigned ldap_desc_size;
-	const char *asn_desc;	/* description in the pkix file if complex type */
-	unsigned int etype;	/* the libtasn1 ASN1_ETYPE or INVALID
-				 * if cannot be simply parsed */
-};
-
 #define ENTRY(oid, ldap, asn, etype) {oid, sizeof(oid)-1, ldap, sizeof(ldap)-1, asn, etype}
 
-/* when there is no ldap description */
+/* when there is no name description */
 #define ENTRY_ND(oid, asn, etype) {oid, sizeof(oid)-1, NULL, 0, asn, etype}
 
 /* This list contains all the OIDs that may be
@@ -136,24 +126,26 @@ static const struct oid_to_string _oid2str[] = {
 	ENTRY("1.2.643.100.1", "OGRN", NULL, ASN1_ETYPE_NUMERIC_STRING),
 	/* Individual insurance account number */
 	ENTRY("1.2.643.100.3", "SNILS", NULL, ASN1_ETYPE_NUMERIC_STRING),
+	/* Main state registration number for individual enterpreneurs */
+	ENTRY("1.2.643.100.5", "OGRNIP", NULL, ASN1_ETYPE_NUMERIC_STRING),
 	/* VAT identification number */
 	ENTRY("1.2.643.3.131.1.1", "INN", NULL, ASN1_ETYPE_NUMERIC_STRING),
 
 	{NULL, 0, NULL, 0, NULL, 0}
 };
 
-static const struct oid_to_string *get_oid_entry(const char *oid)
+const struct oid_to_string *_gnutls_oid_get_entry(const struct oid_to_string *ots, const char *oid)
 {
 	unsigned int i = 0;
 	unsigned len = strlen(oid);
 
 	do {
-		if (len == _oid2str[i].oid_size &&
-			strcmp(_oid2str[i].oid, oid) == 0)
-			return &_oid2str[i];
+		if (len == ots[i].oid_size &&
+			strcmp(ots[i].oid, oid) == 0)
+			return &ots[i];
 		i++;
 	}
-	while (_oid2str[i].oid != NULL);
+	while (ots[i].oid != NULL);
 
 	return NULL;
 }
@@ -163,9 +155,9 @@ const char *_gnutls_ldap_string_to_oid(const char *str, unsigned str_len)
 	unsigned int i = 0;
 
 	do {
-		if ((_oid2str[i].ldap_desc != NULL) &&
-		    (str_len == _oid2str[i].ldap_desc_size) &&
-		    (c_strncasecmp(_oid2str[i].ldap_desc, str, str_len) ==
+		if ((_oid2str[i].name_desc != NULL) &&
+		    (str_len == _oid2str[i].name_desc_size) &&
+		    (c_strncasecmp(_oid2str[i].name_desc, str, str_len) ==
 		     0))
 			return _oid2str[i].oid;
 		i++;
@@ -240,18 +232,7 @@ static int str_escape(const gnutls_datum_t * str, gnutls_datum_t * escaped)
  **/
 int gnutls_x509_dn_oid_known(const char *oid)
 {
-	unsigned int i = 0;
-	unsigned len = strlen(oid);
-
-	do {
-		if (len == _oid2str[i].oid_size &&
-			strcmp(_oid2str[i].oid, oid) == 0)
-			return 1;
-		i++;
-	}
-	while (_oid2str[i].oid != NULL);
-
-	return 0;
+	return _gnutls_oid_get_entry(_oid2str, oid) != NULL;
 }
 
 /**
@@ -270,17 +251,10 @@ int gnutls_x509_dn_oid_known(const char *oid)
  **/
 const char *gnutls_x509_dn_oid_name(const char *oid, unsigned int flags)
 {
-	unsigned int i = 0;
-	unsigned len = strlen(oid);
-
-	do {
-		if ((_oid2str[i].oid_size == len) && 
-			strcmp(_oid2str[i].oid, oid) == 0 && _oid2str[i].ldap_desc != NULL)
-			return _oid2str[i].ldap_desc;
-		i++;
-	}
-	while (_oid2str[i].oid != NULL);
+	const struct oid_to_string *entry =_gnutls_oid_get_entry(_oid2str, oid);
 
+	if (entry && entry->name_desc)
+		return entry->name_desc;
 	if (flags & GNUTLS_X509_DN_OID_RETURN_OID)
 		return oid;
 	else
@@ -448,7 +422,7 @@ _gnutls_x509_dn_to_string(const char *oid, void *value,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	oentry = get_oid_entry(oid);
+	oentry = _gnutls_oid_get_entry(_oid2str, oid);
 	if (oentry == NULL) {	/* unknown OID -> hex */
  unknown_oid:
 		ret = data2hex(value, value_size, str);
@@ -718,7 +692,7 @@ x509_read_value(ASN1_TYPE c, const char *root,
 	}
 
 	if (etype == ASN1_ETYPE_BIT_STRING) {
-		len = (len + 7)/8;
+		len = (len + 7) / 8;
 	}
 
 	tmp = gnutls_malloc((size_t) len + 1);
@@ -736,10 +710,21 @@ x509_read_value(ASN1_TYPE c, const char *root,
 			goto cleanup;
 		}
 
-		if (etype == ASN1_ETYPE_BIT_STRING) {
-			ret->size = (len+7) / 8;
-		} else {
+		switch (etype) {
+		case ASN1_ETYPE_BIT_STRING:
+			ret->size = (len + 7) / 8;
+			break;
+		case ASN1_ETYPE_OBJECT_ID:
+			if (len > 0) {
+				ret->size = len - 1;
+			} else {
+				result = gnutls_assert_val(GNUTLS_E_ASN1_DER_ERROR);
+				goto cleanup;
+			}
+			break;
+		default:
 			ret->size = (unsigned) len;
+			break;
 		}
 	} else {
 		ret->size = 0;
@@ -977,7 +962,7 @@ _gnutls_x509_der_encode_and_copy(ASN1_TYPE src, const char *src_name,
 	return 0;
 }
 
-/* Writes the value of the datum in the given ASN1_TYPE. 
+/* Writes the value of the datum in the given ASN1_TYPE.
  */
 int
 _gnutls_x509_write_value(ASN1_TYPE c, const char *root,
@@ -996,7 +981,7 @@ _gnutls_x509_write_value(ASN1_TYPE c, const char *root,
 	return 0;
 }
 
-/* Writes the value of the datum in the given ASN1_TYPE as a string. 
+/* Writes the value of the datum in the given ASN1_TYPE as a string.
  */
 int
 _gnutls_x509_write_string(ASN1_TYPE c, const char *root,
@@ -1048,7 +1033,7 @@ _asnstr_append_name(char *name, size_t name_size, const char *part1,
 int
 _gnutls_x509_encode_and_copy_PKI_params(ASN1_TYPE dst,
 					const char *dst_name,
-					gnutls_pk_params_st * params)
+					const gnutls_pk_params_st * params)
 {
 	const char *oid;
 	gnutls_datum_t der = { NULL, 0 };
@@ -1116,7 +1101,7 @@ _gnutls_x509_encode_and_copy_PKI_params(ASN1_TYPE dst,
  */
 int
 _gnutls_x509_encode_PKI_params(gnutls_datum_t * der,
-			       gnutls_pk_params_st * params)
+			       const gnutls_pk_params_st * params)
 {
 	int ret;
 	ASN1_TYPE tmp;
@@ -1187,7 +1172,7 @@ _gnutls_x509_get_pk_algorithm(ASN1_TYPE src, const char *src_name,
 		return algo;
 	}
 
-	/* Now read the parameters' bits 
+	/* Now read the parameters' bits
 	 */
 	if (lcurve != GNUTLS_ECC_CURVE_INVALID) { /* curve present */
 		bits[0] = gnutls_ecc_curve_get_size(lcurve)*8;
@@ -1326,7 +1311,7 @@ _gnutls_x509_get_signature(ASN1_TYPE src, const char *src_name,
 	signature->data = NULL;
 	signature->size = 0;
 
-	/* Read the signature 
+	/* Read the signature
 	 */
 	len = 0;
 	result = asn1_read_value(src, src_name, NULL, &len);
@@ -1467,7 +1452,7 @@ _gnutls_x509_encode_and_write_attribute(const char *given_oid,
 	int result;
 	const struct oid_to_string *oentry;
 
-	oentry = get_oid_entry(given_oid);
+	oentry = _gnutls_oid_get_entry(_oid2str, given_oid);
 	if (oentry == NULL) {
 		gnutls_assert();
 		_gnutls_debug_log("Cannot find OID: %s\n", given_oid);
@@ -1562,7 +1547,7 @@ int _gnutls_strdatum_to_buf(gnutls_datum_t * d, void *buf,
 }
 
 int
-_gnutls_x509_get_raw_field2(ASN1_TYPE c2, gnutls_datum_t * raw,
+_gnutls_x509_get_raw_field2(ASN1_TYPE c2, const gnutls_datum_t * raw,
 			 const char *whom, gnutls_datum_t * dn)
 {
 	int result, len1;
@@ -1587,7 +1572,7 @@ _gnutls_x509_get_raw_field2(ASN1_TYPE c2, gnutls_datum_t * raw,
 	return result;
 }
 
-int _gnutls_copy_string(gnutls_datum_t* str, uint8_t *out, size_t *out_size)
+int _gnutls_copy_string(const gnutls_datum_t* str, uint8_t *out, size_t *out_size)
 {
 unsigned size_to_check;
 
@@ -1610,7 +1595,7 @@ unsigned size_to_check;
 	return 0;
 }
 
-int _gnutls_copy_data(gnutls_datum_t* str, uint8_t *out, size_t *out_size)
+int _gnutls_copy_data(const gnutls_datum_t* str, uint8_t *out, size_t *out_size)
 {
 	if ((unsigned) str->size > *out_size) {
 		gnutls_assert();
@@ -1682,7 +1667,7 @@ int _gnutls_x509_raw_crt_to_raw_pubkey(const gnutls_datum_t * cert,
 }
 
 unsigned
-_gnutls_check_valid_key_id(gnutls_datum_t *key_id,
+_gnutls_check_valid_key_id(const gnutls_datum_t *key_id,
 			   gnutls_x509_crt_t cert, time_t now,
 			   unsigned *has_ski)
 {
@@ -1716,38 +1701,52 @@ _gnutls_check_valid_key_id(gnutls_datum_t *key_id,
 	return result;
 }
 
-/* Takes a certificate list and orders it with subject, issuer order.
+/* Sort a certficate list in place with subject, issuer order. @clist_size must
+ * be equal to or less than %DEFAULT_MAX_VERIFY_DEPTH.
  *
- * *clist_size contains the size of the ordered list (which is always less or
- * equal to the original).
- * @func: the function to call to elements outside the sort.
+ * Returns the index in clist where the initial contiguous segment ends. If the
+ * chain contains multiple segments separated by gaps (e.g., missing issuers),
+ * the caller shall call this function multiple times.
  *
- * This function is intentionally kept simple to be easily verified
- * so that it can be used with untrusted chains. The introduction
- * of the func parameter added significant complexity in that aspect.
- * If more demanding use-cases need to be handled, consider splitting
- * that function.
+ * For example, suppose we want the following certificate chain as a result of
+ * sorting:
  *
- * Returns the sorted list which may be the original clist.
+ *   A -> (B) -> C -> D -> E -> (F) -> G -> H -> I
  *
+ * from the input [A, C, E, D, G, I, H] (B and F are missing).
+ *
+ * On the first call of this function:
+ *
+ *   _gnutls_sort_clist(clist, clist_size)
+ *
+ * it returns 1, meaning that the first segment only contains A.  The content of
+ * @clist will remain the same [A, C, E, D, G, I, H].
+ *
+ * Then the caller will call this function again, starting from the second
+ * element:
+ *
+ *   _gnutls_sort_clist(&clist[1], clist_size - 1)
+ *
+ * This time it returns 3, meaning that the first segment contains [C, D, E].
+ * The content of @clist will be [A, C, D, E, G, I, H].
+ *
+ * Finally, after calling the function on the remaining elements:
+ *
+ *   _gnutls_sort_clist(&clist[1 + 3], clist_size - (1 + 3))
+ *
+ * It will return 3, meaning that the first segment contains [G, H, I].  At this
+ * point, sorting of @clist is complete.
  */
-gnutls_x509_crt_t *_gnutls_sort_clist(gnutls_x509_crt_t
-				     sorted[DEFAULT_MAX_VERIFY_DEPTH],
-				     gnutls_x509_crt_t *clist,
-				     unsigned int *clist_size,
-				     gnutls_cert_vfunc func)
+unsigned int _gnutls_sort_clist(gnutls_x509_crt_t *clist,
+				unsigned int clist_size)
 {
 	int prev;
-	unsigned int j, i;
+	unsigned int i, j, k;
 	int issuer[DEFAULT_MAX_VERIFY_DEPTH];	/* contain the index of the issuers */
 	bool insorted[DEFAULT_MAX_VERIFY_DEPTH]; /* non zero if clist[i] used in sorted list */
-	unsigned orig_size = *clist_size;
+	gnutls_x509_crt_t sorted[DEFAULT_MAX_VERIFY_DEPTH];
 
-	/* Do not bother sorting if too many certificates are given.
-	 * Prevent any DoS attacks.
-	 */
-	if (*clist_size > DEFAULT_MAX_VERIFY_DEPTH)
-		return clist;
+	assert(clist_size <= DEFAULT_MAX_VERIFY_DEPTH);
 
 	for (i = 0; i < DEFAULT_MAX_VERIFY_DEPTH; i++) {
 		issuer[i] = -1;
@@ -1758,78 +1757,72 @@ gnutls_x509_crt_t *_gnutls_sort_clist(gnutls_x509_crt_t
 	 * in issuer array. O(n^2) so consider that before
 	 * increasing DEFAULT_MAX_VERIFY_DEPTH.
 	 */
-	for (i = 0; i < *clist_size; i++) {
-		for (j = 1; j < *clist_size; j++) {
+	for (i = 0; i < clist_size; i++) {
+		/* Self-signed certificate found in the chain; skip it
+		 * as it should only appear in the trusted set.
+		 */
+		if (gnutls_x509_crt_check_issuer(clist[i], clist[i])) {
+			_gnutls_cert_log("self-signed cert found", clist[i]);
+			continue;
+		}
+
+		for (j = 1; j < clist_size; j++) {
 			if (i == j)
 				continue;
 
-			if (gnutls_x509_crt_check_issuer(clist[i],
-							 clist[j]) != 0) {
+			if (gnutls_x509_crt_check_issuer(clist[i], clist[j])) {
 				issuer[i] = j;
 				break;
 			}
 		}
 	}
 
-	/* the first element is always included */
 	sorted[0] = clist[0];
 	insorted[0] = 1;
 
-	if (issuer[0] == -1) {
-		*clist_size = 1;
-		goto exit;
-	}
-
 	prev = 0;
-	for (i = 1; i < *clist_size; i++) {
+	for (i = 1; i < clist_size; i++) {
 		prev = issuer[prev];
 		if (prev < 0) {	/* no issuer */
-			*clist_size = i;
 			break;
 		}
+
 		sorted[i] = clist[prev];
 		insorted[prev] = 1;
 	}
 
- exit:
-	if (func) {
-		/* call func() on all the elements that were
-		 * not used in the sorted list */
-		for (i = 1; i < orig_size; i++) {
-			if (insorted[i] == 0) {
-				func(clist[i]);
-			}
+	/* append the remaining certs */
+	for (j = 1, k = i; j < clist_size; j++) {
+		if (!insorted[j]) {
+			sorted[k++] = clist[j];
 		}
 	}
 
-	return sorted;
+	/* write out the sorted list */
+	assert(k == clist_size);
+	for (j = 0; j < clist_size; j++) {
+		clist[j] = sorted[j];
+	}
+
+	return i;
 }
 
 int _gnutls_check_if_sorted(gnutls_x509_crt_t * crt, int nr)
 {
-	void *prev_dn = NULL;
-	void *dn;
-	size_t prev_dn_size = 0, dn_size;
 	int i, ret;
 
 	/* check if the X.509 list is ordered */
 	if (nr > 1) {
 		for (i = 0; i < nr; i++) {
 			if (i > 0) {
-				dn = crt[i]->raw_dn.data;
-				dn_size = crt[i]->raw_dn.size;
-
-				if (dn_size != prev_dn_size
-				    || memcmp(dn, prev_dn, dn_size) != 0) {
+				if (!_gnutls_x509_compare_raw_dn(&crt[i]->raw_dn,
+				                                 &crt[i-1]->raw_issuer_dn)) {
 					ret =
 					    gnutls_assert_val
 					    (GNUTLS_E_CERTIFICATE_LIST_UNSORTED);
 					goto cleanup;
 				}
 			}
-
-			prev_dn = crt[i]->raw_issuer_dn.data;
-			prev_dn_size = crt[i]->raw_issuer_dn.size;
 		}
 	}
 	ret = 0;
@@ -1924,3 +1917,23 @@ gnutls_gost_paramset_t gnutls_oid_to_gost_paramset(const char *oid)
 	else
 		return gnutls_assert_val(GNUTLS_GOST_PARAMSET_UNKNOWN);
 }
+
+int _gnutls_x509_get_version(ASN1_TYPE root, const char *name)
+{
+	uint8_t version[8];
+	int len, result;
+
+	len = sizeof(version);
+	result = asn1_read_value(root, name, version, &len);
+	if (result != ASN1_SUCCESS) {
+		if (result == ASN1_ELEMENT_NOT_FOUND)
+			return 1;	/* the DEFAULT version */
+		gnutls_assert();
+		return _gnutls_asn2err(result);
+	}
+
+	if (len != 1 || version[0] >= 0x80)
+		return gnutls_assert_val(GNUTLS_E_ASN1_DER_ERROR);
+
+	return (int) version[0] + 1;
+}
