@@ -300,7 +300,7 @@ gnutls_x509_crl_get_dn_oid(gnutls_x509_crl_t crl,
 /**
  * gnutls_x509_crl_get_issuer_dn2:
  * @crl: should contain a #gnutls_x509_crl_t type
- * @dn: a pointer to a structure to hold the name
+ * @dn: a pointer to a structure to hold the name; must be freed using gnutls_free()
  *
  * This function will allocate buffer and copy the name of the CRL issuer.
  * The name will be in the form "C=xxxx,O=yyyy,CN=zzzz" as
@@ -331,7 +331,7 @@ gnutls_x509_crl_get_issuer_dn2(gnutls_x509_crl_t crl, gnutls_datum_t * dn)
 /**
  * gnutls_x509_crl_get_issuer_dn3:
  * @crl: should contain a #gnutls_x509_crl_t type
- * @dn: a pointer to a structure to hold the name
+ * @dn: a pointer to a structure to hold the name; must be freed using gnutls_free()
  * @flags: zero or %GNUTLS_X509_DN_FLAG_COMPAT
  *
  * This function will allocate buffer and copy the name of the CRL issuer.
@@ -484,23 +484,12 @@ gnutls_x509_crl_get_signature(gnutls_x509_crl_t crl,
  **/
 int gnutls_x509_crl_get_version(gnutls_x509_crl_t crl)
 {
-	uint8_t version[8];
-	int len, result;
-
 	if (crl == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	len = sizeof(version);
-	if ((result =
-	     asn1_read_value(crl->crl, "tbsCertList.version", version,
-			     &len)) != ASN1_SUCCESS) {
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	return (int) version[0] + 1;
+	return _gnutls_x509_get_version(crl->crl, "tbsCertList.version");
 }
 
 /**
