@@ -15,7 +15,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -35,7 +35,7 @@
 #define addf _gnutls_buffer_append_printf
 #define adds _gnutls_buffer_append_str
 
-static void print_req(gnutls_buffer_st * str, gnutls_ocsp_req_t req)
+static void print_req(gnutls_buffer_st * str, gnutls_ocsp_req_const_t req)
 {
 	int ret;
 	unsigned indx;
@@ -109,7 +109,7 @@ static void print_req(gnutls_buffer_st * str, gnutls_ocsp_req_t req)
 		if (indx == 0)
 			adds(str, "\tExtensions:\n");
 
-		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) &&
+		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) - 1 &&
 		    memcmp(oid.data, GNUTLS_OCSP_NONCE, oid.size) == 0) {
 			gnutls_datum_t nonce;
 			unsigned int ncrit;
@@ -169,7 +169,7 @@ static void print_req(gnutls_buffer_st * str, gnutls_ocsp_req_t req)
  *   negative error value.
  **/
 int
-gnutls_ocsp_req_print(gnutls_ocsp_req_t req,
+gnutls_ocsp_req_print(gnutls_ocsp_req_const_t req,
 		      gnutls_ocsp_print_formats_t format,
 		      gnutls_datum_t * out)
 {
@@ -197,7 +197,7 @@ gnutls_ocsp_req_print(gnutls_ocsp_req_t req,
 }
 
 static void
-print_resp(gnutls_buffer_st * str, gnutls_ocsp_resp_t resp,
+print_resp(gnutls_buffer_st * str, gnutls_ocsp_resp_const_t resp,
 	   gnutls_ocsp_print_formats_t format)
 {
 	int ret;
@@ -254,7 +254,7 @@ print_resp(gnutls_buffer_st * str, gnutls_ocsp_resp_t resp,
 		adds(str, "\tResponse Type: ");
 #define OCSP_BASIC "1.3.6.1.5.5.7.48.1.1"
 
-		if (oid.size == sizeof(OCSP_BASIC)
+		if (oid.size == sizeof(OCSP_BASIC) - 1
 		    && memcmp(oid.data, OCSP_BASIC, oid.size) == 0) {
 			adds(str, "Basic OCSP Response\n");
 			gnutls_free(oid.data);
@@ -473,7 +473,7 @@ print_resp(gnutls_buffer_st * str, gnutls_ocsp_resp_t resp,
 			continue;
 		}
 
-		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) &&
+		if (oid.size == sizeof(GNUTLS_OCSP_NONCE) - 1 &&
 		    memcmp(oid.data, GNUTLS_OCSP_NONCE, oid.size) == 0) {
 			gnutls_datum_t nonce;
 			unsigned int ncrit;
@@ -639,7 +639,7 @@ print_resp(gnutls_buffer_st * str, gnutls_ocsp_resp_t resp,
  *   negative error value.
  **/
 int
-gnutls_ocsp_resp_print(gnutls_ocsp_resp_t resp,
+gnutls_ocsp_resp_print(gnutls_ocsp_resp_const_t resp,
 		       gnutls_ocsp_print_formats_t format,
 		       gnutls_datum_t * out)
 {
