@@ -17,20 +17,21 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
 /* Functions that relate on PKCS12 Bag packet parsing.
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <common.h>
 #include "x509_int.h"
+#include "pkcs7_int.h"
 
 /**
  * gnutls_pkcs12_bag_init:
@@ -55,13 +56,12 @@ int gnutls_pkcs12_bag_init(gnutls_pkcs12_bag_t * bag)
 
 static inline void _pkcs12_bag_free_data(gnutls_pkcs12_bag_t bag)
 {
-	int i;
+	unsigned i;
 
 	for (i = 0; i < bag->bag_elements; i++) {
 		_gnutls_free_datum(&bag->element[i].data);
 		_gnutls_free_datum(&bag->element[i].local_key_id);
 		gnutls_free(bag->element[i].friendly_name);
-		bag->element[i].friendly_name = NULL;
 		bag->element[i].type = 0;
 	}
 
@@ -91,10 +91,10 @@ void gnutls_pkcs12_bag_deinit(gnutls_pkcs12_bag_t bag)
  *
  * This function will return the bag's type.
  *
- * Returns: One of the #gnutls_pkcs12_bag_type_t enumerations.
+ * Returns: On error a negative error value or one of the #gnutls_pkcs12_bag_type_t enumerations.
  **/
-gnutls_pkcs12_bag_type_t
-gnutls_pkcs12_bag_get_type(gnutls_pkcs12_bag_t bag, int indx)
+int
+gnutls_pkcs12_bag_get_type(gnutls_pkcs12_bag_t bag, unsigned indx)
 {
 	if (bag == NULL) {
 		gnutls_assert();
@@ -139,7 +139,7 @@ int gnutls_pkcs12_bag_get_count(gnutls_pkcs12_bag_t bag)
  *   negative error value.
  **/
 int
-gnutls_pkcs12_bag_get_data(gnutls_pkcs12_bag_t bag, int indx,
+gnutls_pkcs12_bag_get_data(gnutls_pkcs12_bag_t bag, unsigned indx,
 			   gnutls_datum_t * data)
 {
 	if (bag == NULL) {
@@ -516,7 +516,7 @@ gnutls_pkcs12_bag_set_crl(gnutls_pkcs12_bag_t bag, gnutls_x509_crl_t crl)
  *   negative error value. or a negative error code on error.
  **/
 int
-gnutls_pkcs12_bag_set_key_id(gnutls_pkcs12_bag_t bag, int indx,
+gnutls_pkcs12_bag_set_key_id(gnutls_pkcs12_bag_t bag, unsigned indx,
 			     const gnutls_datum_t * id)
 {
 	int ret;
@@ -557,7 +557,7 @@ gnutls_pkcs12_bag_set_key_id(gnutls_pkcs12_bag_t bag, int indx,
  *   negative error value. or a negative error code on error.
  **/
 int
-gnutls_pkcs12_bag_get_key_id(gnutls_pkcs12_bag_t bag, int indx,
+gnutls_pkcs12_bag_get_key_id(gnutls_pkcs12_bag_t bag, unsigned indx,
 			     gnutls_datum_t * id)
 {
 	if (bag == NULL) {
@@ -590,7 +590,7 @@ gnutls_pkcs12_bag_get_key_id(gnutls_pkcs12_bag_t bag, int indx,
  *   negative error value. or a negative error code on error.
  **/
 int
-gnutls_pkcs12_bag_get_friendly_name(gnutls_pkcs12_bag_t bag, int indx,
+gnutls_pkcs12_bag_get_friendly_name(gnutls_pkcs12_bag_t bag, unsigned indx,
 				    char **name)
 {
 	if (bag == NULL) {
@@ -624,7 +624,7 @@ gnutls_pkcs12_bag_get_friendly_name(gnutls_pkcs12_bag_t bag, int indx,
  *   negative error value. or a negative error code on error.
  **/
 int
-gnutls_pkcs12_bag_set_friendly_name(gnutls_pkcs12_bag_t bag, int indx,
+gnutls_pkcs12_bag_set_friendly_name(gnutls_pkcs12_bag_t bag, unsigned indx,
 				    const char *name)
 {
 	if (bag == NULL) {
@@ -810,7 +810,7 @@ gnutls_pkcs12_bag_enc_info(gnutls_pkcs12_bag_t bag, unsigned int *schema, unsign
 {
 	int ret;
 	struct pbkdf2_params kdf;
-	const struct pbes2_schema_st *p;
+	const struct pkcs_cipher_schema_st *p;
 
 	if (bag == NULL) {
 		gnutls_assert();
