@@ -135,10 +135,9 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 
 	gnutls_pk_params_init(&pkey->params);
 
-	if ((result =
-	     asn1_create_element(_gnutls_get_gnutls_asn(),
-				 "GNUTLS.RSAPrivateKey",
-				 &pkey_asn)) != ASN1_SUCCESS) {
+	if (asn1_create_element(_gnutls_get_gnutls_asn(),
+				"GNUTLS.RSAPrivateKey",
+				&pkey_asn) != ASN1_SUCCESS) {
 		gnutls_assert();
 		return NULL;
 	}
@@ -151,65 +150,58 @@ _gnutls_privkey_decode_pkcs1_rsa_key(const gnutls_datum_t * raw_key,
 		goto error;
 	}
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "modulus",
-					    &pkey->params.params[0])) < 0)
+	if (_gnutls_x509_read_int(pkey_asn, "modulus",
+				  &pkey->params.params[0]) < 0)
 	{
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result =
-	     _gnutls_x509_read_int(pkey_asn, "publicExponent",
-				   &pkey->params.params[1])) < 0) {
+	if (_gnutls_x509_read_int(pkey_asn, "publicExponent",
+				  &pkey->params.params[1]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result =
-	     _gnutls_x509_read_key_int(pkey_asn, "privateExponent",
-				   &pkey->params.params[2])) < 0) {
+	if (_gnutls_x509_read_key_int(pkey_asn, "privateExponent",
+				      &pkey->params.params[2]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_key_int(pkey_asn, "prime1",
-					    &pkey->params.params[3])) < 0)
-	{
+	if (_gnutls_x509_read_key_int(pkey_asn, "prime1",
+				      &pkey->params.params[3]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_key_int(pkey_asn, "prime2",
-					    &pkey->params.params[4])) < 0)
-	{
+	if (_gnutls_x509_read_key_int(pkey_asn, "prime2",
+				      &pkey->params.params[4]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_key_int(pkey_asn, "coefficient",
-					    &pkey->params.params[5])) < 0)
-	{
+	if (_gnutls_x509_read_key_int(pkey_asn, "coefficient",
+				      &pkey->params.params[5]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_key_int(pkey_asn, "exponent1",
-					    &pkey->params.params[6])) < 0)
-	{
+	if (_gnutls_x509_read_key_int(pkey_asn, "exponent1",
+				      &pkey->params.params[6]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_key_int(pkey_asn, "exponent2",
-					    &pkey->params.params[7])) < 0)
-	{
+	if (_gnutls_x509_read_key_int(pkey_asn, "exponent2",
+				      &pkey->params.params[7]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
@@ -353,10 +345,9 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 	char oid[MAX_OID_SIZE];
 	int oid_size;
 
-	if ((result =
-	     asn1_create_element(_gnutls_get_gnutls_asn(),
-				 "GNUTLS.DSAPrivateKey",
-				 &dsa_asn)) != ASN1_SUCCESS) {
+	if (asn1_create_element(_gnutls_get_gnutls_asn(),
+				"GNUTLS.DSAPrivateKey",
+				&dsa_asn) != ASN1_SUCCESS) {
 		gnutls_assert();
 		return NULL;
 	}
@@ -372,40 +363,36 @@ decode_dsa_key(const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 		goto error;
 	}
 
-	if ((result =
-	     _gnutls_x509_read_int(dsa_asn, "p",
-				   &pkey->params.params[0])) < 0) {
+	if (_gnutls_x509_read_int(dsa_asn, "p",
+				  &pkey->params.params[0]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result =
-	     _gnutls_x509_read_int(dsa_asn, "q",
-				   &pkey->params.params[1])) < 0) {
+	if (_gnutls_x509_read_int(dsa_asn, "q",
+				  &pkey->params.params[1]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result =
-	     _gnutls_x509_read_int(dsa_asn, "g",
-				   &pkey->params.params[2])) < 0) {
+	if (_gnutls_x509_read_int(dsa_asn, "g",
+				  &pkey->params.params[2]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result =
-	     _gnutls_x509_read_int(dsa_asn, "Y",
-				   &pkey->params.params[3])) < 0) {
+	if (_gnutls_x509_read_int(dsa_asn, "Y",
+				  &pkey->params.params[3]) < 0) {
 		gnutls_assert();
 		goto error;
 	}
 	pkey->params.params_nr++;
 
-	if ((result = _gnutls_x509_read_key_int(dsa_asn, "priv",
-					    &pkey->params.params[4])) < 0)
+	if (_gnutls_x509_read_key_int(dsa_asn, "priv",
+				      &pkey->params.params[4]) < 0)
 	{
 		gnutls_assert();
 		goto error;
@@ -885,7 +872,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	gnutls_pk_params_init(&key->params);
 
 	siz = m->size;
-	if (_gnutls_mpi_init_scan_nz(&key->params.params[0], m->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[RSA_MODULUS], m->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -893,23 +880,25 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	key->params.params_nr++;
 
 	siz = e->size;
-	if (_gnutls_mpi_init_scan_nz(&key->params.params[1], e->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[RSA_PUB], e->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
 	}
 	key->params.params_nr++;
 
-	siz = d->size;
-	if (_gnutls_mpi_init_scan_nz(&key->params.params[2], d->data, siz)) {
-		gnutls_assert();
-		ret = GNUTLS_E_MPI_SCAN_FAILED;
-		goto cleanup;
+	if (d) {
+		siz = d->size;
+		if (_gnutls_mpi_init_scan_nz(&key->params.params[RSA_PRIV], d->data, siz)) {
+			gnutls_assert();
+			ret = GNUTLS_E_MPI_SCAN_FAILED;
+			goto cleanup;
+		}
+		key->params.params_nr++;
 	}
-	key->params.params_nr++;
 
 	siz = p->size;
-	if (_gnutls_mpi_init_scan_nz(&key->params.params[3], p->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[RSA_PRIME1], p->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -917,7 +906,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
 	key->params.params_nr++;
 
 	siz = q->size;
-	if (_gnutls_mpi_init_scan_nz(&key->params.params[4], q->data, siz)) {
+	if (_gnutls_mpi_init_scan_nz(&key->params.params[RSA_PRIME2], q->data, siz)) {
 		gnutls_assert();
 		ret = GNUTLS_E_MPI_SCAN_FAILED;
 		goto cleanup;
@@ -988,7 +977,7 @@ gnutls_x509_privkey_import_rsa_raw2(gnutls_x509_privkey_t key,
  * @p: holds the p
  * @q: holds the q
  * @g: holds the g
- * @y: holds the y
+ * @y: holds the y (optional)
  * @x: holds the x
  *
  * This function will convert the given DSA raw parameters to the
@@ -1037,11 +1026,13 @@ gnutls_x509_privkey_import_dsa_raw(gnutls_x509_privkey_t key,
 		goto cleanup;
 	}
 
-	siz = y->size;
-	if (_gnutls_mpi_init_scan_nz(&key->params.params[3], y->data, siz)) {
-		gnutls_assert();
-		ret = GNUTLS_E_MPI_SCAN_FAILED;
-		goto cleanup;
+	if (y) {
+		siz = y->size;
+		if (_gnutls_mpi_init_scan_nz(&key->params.params[3], y->data, siz)) {
+			gnutls_assert();
+			ret = GNUTLS_E_MPI_SCAN_FAILED;
+			goto cleanup;
+		}
 	}
 
 	siz = x->size;
