@@ -65,6 +65,12 @@ extern int ask_pass;
 #define MAX_ENTRIES 128
 #define MAX_POLICIES 8
 
+#define CHECK_MALLOC(x) \
+	if (x == NULL) { \
+		fprintf(stderr, "memory error\n"); \
+		exit(1); \
+	}
+
 #define PRINT_TIME_T_ERROR \
 	if (sizeof(time_t) < 8) \
 		fprintf(stderr, "This system expresses time with a 32-bit time_t; that prevents dates after 2038 to be expressed by GnuTLS.\n")
@@ -245,6 +251,7 @@ void cfg_init(void)
     if (s_name == NULL) { \
       i = 0; \
       s_name = malloc(sizeof(char*)*MAX_ENTRIES); \
+      CHECK_MALLOC(s_name); \
       do { \
 	if (val && strcmp(val->pzName, name)!=0) \
 	  continue; \
@@ -266,10 +273,12 @@ void cfg_init(void)
     if (s_name == NULL) { \
       i = 0; \
       s_name = malloc(sizeof(char*)*MAX_ENTRIES); \
+      CHECK_MALLOC(s_name); \
       do { \
 	if (val && strcmp(val->pzName, name)!=0) \
 	  continue; \
 	str = strdup(val->v.strVal); \
+	CHECK_MALLOC(str); \
 	if ((p=strchr(str, ' ')) == NULL && (p=strchr(str, '\t')) == NULL) { \
 	  fprintf(stderr, "Error parsing %s\n", name); \
 	  exit(1); \
@@ -651,6 +660,8 @@ read_crq_set(gnutls_x509_crq_t crq, const char *input_str, const char *oid)
 	free(lineptr);
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 /* The input_str should contain %d or %u to print the default.
  */
 static int64_t read_int_with_default(const char *input_str, long def)
@@ -705,6 +716,7 @@ static int64_t read_int_with_default(const char *input_str, long def)
 
 	return l;
 }
+#pragma GCC diagnostic pop
 
 int64_t read_int(const char *input_str)
 {
@@ -1871,10 +1883,11 @@ int get_tls_server_status(void)
 /* convert a printable IP to binary */
 static int string_to_ip(unsigned char *ip, const char *str)
 {
-	int len = strlen(str);
 	int ret;
 
 #if HAVE_IPV6
+	int len = strlen(str);
+
 	if (strchr(str, ':') != NULL || len > 16) {	/* IPv6 */
 		ret = inet_pton(AF_INET6, str, ip);
 		if (ret <= 0) {
@@ -2541,8 +2554,8 @@ int get_cert_sign_status(void)
 	} else {
 		return
 		    read_yesno
-		    ("Will the certificate be used to sign other certificates? (y/N): ",
-		     0);
+		    ("Will the certificate be used to sign other certificates? (Y/n): ",
+		     1);
 	}
 }
 
