@@ -1,5 +1,5 @@
 /* Sequential list data type implemented by a linked list.
-   Copyright (C) 2006-2021 Free Software Foundation, Inc.
+   Copyright (C) 2006-2020 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2006.
 
    This program is free software: you can redistribute it and/or modify
@@ -229,24 +229,6 @@ gl_linked_previous_node (gl_list_t list, gl_list_node_t node)
   return (node->prev != &list->root ? node->prev : NULL);
 }
 
-static gl_list_node_t _GL_ATTRIBUTE_PURE
-gl_linked_first_node (gl_list_t list)
-{
-  if (list->count > 0)
-    return list->root.next;
-  else
-    return NULL;
-}
-
-static gl_list_node_t _GL_ATTRIBUTE_PURE
-gl_linked_last_node (gl_list_t list)
-{
-  if (list->count > 0)
-    return list->root.prev;
-  else
-    return NULL;
-}
-
 static const void * _GL_ATTRIBUTE_PURE
 gl_linked_get_at (gl_list_t list, size_t position)
 {
