@@ -1,9 +1,9 @@
 /* Test of parse_datetime() function.
-   Copyright (C) 2008-2025 Free Software Foundation, Inc.
+   Copyright (C) 2008-2020 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -49,7 +49,7 @@ static const char *const day_table[] =
 };
 
 
-#if ! HAVE_STRUCT_TM_TM_GMTOFF
+#if ! HAVE_TM_GMTOFF
 /* Shift A right by B bits portably, by dividing A by 2**B and
    truncating towards minus infinity.  A and B should be free of side
    effects, and B should be in the range 0 <= B <= INT_BITS - 2, where
@@ -91,14 +91,14 @@ tm_diff (struct tm const *a, struct tm const *b)
                 + (a->tm_min - b->tm_min))
           + (a->tm_sec - b->tm_sec));
 }
-#endif
+#endif /* ! HAVE_TM_GMTOFF */
 
 static long
 gmt_offset (time_t s)
 {
   long gmtoff;
 
-#if !HAVE_STRUCT_TM_TM_GMTOFF
+#if !HAVE_TM_GMTOFF
   struct tm tm_local = *localtime (&s);
   struct tm tm_gmt   = *gmtime (&s);
 
@@ -110,19 +110,8 @@ gmt_offset (time_t s)
   return gmtoff;
 }
 
-/* Define SOME_TIMEPOINT to some tv_sec value that is supported by the
-   platform's localtime() function and that is on the same weekday as
-   the Unix epoch.  */
-#if defined _WIN32 && !defined __CYGWIN__
-/* On native Windows, localtime() fails for all time_t values < 0.  */
-# define SOME_TIMEPOINT (700 * 86400)
-#else
-/* The Unix epoch.  */
-# define SOME_TIMEPOINT 0
-#endif
-
 int
-main (_GL_UNUSED int argc, char **argv)
+main (int argc _GL_UNUSED, char **argv)
 {
   struct timespec result;
   struct timespec result2;
@@ -137,8 +126,7 @@ main (_GL_UNUSED int argc, char **argv)
      should disable any leap second support.  Otherwise, there will be
      a problem with glibc on sites that default to leap seconds; see
      <https://bugs.gnu.org/12206>.  */
-  ASSERT (setenv ("TZ", "EST5EDT,M3.2.0,M11.1.0", 1) == 0);
-  tzset ();
+  setenv ("TZ", "EST5EDT,M3.2.0,M11.1.0", 1);
 
   gmtoff = gmt_offset (ref_time);
 
@@ -148,7 +136,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01T11:55:18";
   expected.tv_sec = ref_time - gmtoff;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -158,17 +146,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01 11:55:18";
   expected.tv_sec = ref_time - gmtoff;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
-  LOG (p, expected, result);
-  ASSERT (expected.tv_sec == result.tv_sec
-          && expected.tv_nsec == result.tv_nsec);
-
-  /* ISO 8601 extended date and time of day representation,
-     ' ' separator, 'J' (local) time zone */
-  p = "2011-05-01 11:55:18J";
-  expected.tv_sec = ref_time - gmtoff;
-  expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -179,7 +157,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01T11:55:18Z";
   expected.tv_sec = ref_time;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -189,7 +167,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01 11:55:18Z";
   expected.tv_sec = ref_time;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -200,7 +178,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01T11:55:18-07:00";
   expected.tv_sec = 1304276118;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -210,7 +188,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01 11:55:18-07:00";
   expected.tv_sec = 1304276118;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -221,7 +199,7 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01T11:55:18-07";
   expected.tv_sec = 1304276118;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
@@ -231,20 +209,20 @@ main (_GL_UNUSED int argc, char **argv)
   p = "2011-05-01 11:55:18-07";
   expected.tv_sec = 1304276118;
   expected.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, NULL));
+  ASSERT (parse_datetime (&result, p, 0));
   LOG (p, expected, result);
   ASSERT (expected.tv_sec == result.tv_sec
           && expected.tv_nsec == result.tv_nsec);
 
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "now";
   ASSERT (parse_datetime (&result, p, &now));
   LOG (p, now, result);
   ASSERT (now.tv_sec == result.tv_sec && now.tv_nsec == result.tv_nsec);
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "tomorrow";
   ASSERT (parse_datetime (&result, p, &now));
@@ -252,7 +230,7 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (now.tv_sec + 24 * 60 * 60 == result.tv_sec
           && now.tv_nsec == result.tv_nsec);
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "yesterday";
   ASSERT (parse_datetime (&result, p, &now));
@@ -260,7 +238,7 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (now.tv_sec - 24 * 60 * 60 == result.tv_sec
           && now.tv_nsec == result.tv_nsec);
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "4 hours";
   ASSERT (parse_datetime (&result, p, &now));
@@ -269,7 +247,7 @@ main (_GL_UNUSED int argc, char **argv)
           && now.tv_nsec == result.tv_nsec);
 
   /* test if timezone is not being ignored for day offset */
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+400 +24 hours";
   ASSERT (parse_datetime (&result, p, &now));
@@ -281,7 +259,7 @@ main (_GL_UNUSED int argc, char **argv)
           && result.tv_nsec == result2.tv_nsec);
 
   /* test if several time zones formats are handled same way */
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+14:00";
   ASSERT (parse_datetime (&result, p, &now));
@@ -297,7 +275,7 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (result.tv_sec == result2.tv_sec
           && result.tv_nsec == result2.tv_nsec);
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC-14:00";
   ASSERT (parse_datetime (&result, p, &now));
@@ -313,7 +291,7 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (result.tv_sec == result2.tv_sec
           && result.tv_nsec == result2.tv_nsec);
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+0:15";
   ASSERT (parse_datetime (&result, p, &now));
@@ -324,7 +302,7 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (result.tv_sec == result2.tv_sec
           && result.tv_nsec == result2.tv_nsec);
 
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC-1:30";
   ASSERT (parse_datetime (&result, p, &now));
@@ -337,13 +315,13 @@ main (_GL_UNUSED int argc, char **argv)
 
 
   /* TZ out of range should cause parse_datetime failure */
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+25:00";
   ASSERT (!parse_datetime (&result, p, &now));
 
-  /* Check for several invalid countable dayshifts */
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+        /* Check for several invalid countable dayshifts */
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+4:00 +40 yesterday";
   ASSERT (!parse_datetime (&result, p, &now));
@@ -361,7 +339,7 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (!parse_datetime (&result, p, &now));
 
   /* And check correct usage of dayshifts */
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+400 tomorrow";
   ASSERT (parse_datetime (&result, p, &now));
@@ -376,7 +354,7 @@ main (_GL_UNUSED int argc, char **argv)
   LOG (p, now, result2);
   ASSERT (result.tv_sec == result2.tv_sec
           && result.tv_nsec == result2.tv_nsec);
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+400 yesterday";
   ASSERT (parse_datetime (&result, p, &now));
@@ -386,7 +364,7 @@ main (_GL_UNUSED int argc, char **argv)
   LOG (p, now, result2);
   ASSERT (result.tv_sec == result2.tv_sec
           && result.tv_nsec == result2.tv_nsec);
-  now.tv_sec = SOME_TIMEPOINT + 4711;
+  now.tv_sec = 4711;
   now.tv_nsec = 1267;
   p = "UTC+400 now";
   ASSERT (parse_datetime (&result, p, &now));
@@ -397,40 +375,11 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (result.tv_sec == result2.tv_sec
           && result.tv_nsec == result2.tv_nsec);
 
-  /* If this platform has TZDB, check for GNU Bug#48085.  */
-#if defined _WIN32 && !defined __CYGWIN__
-  ASSERT (setenv ("TZ", "US Eastern Standard Time", 1) == 0);
-#else
-  ASSERT (setenv ("TZ", "America/Indiana/Indianapolis", 1) == 0);
-#endif
-  tzset ();
-  now.tv_sec = 1619641490;
-  now.tv_nsec = 0;
-  struct tm *tm = localtime (&now.tv_sec);
-  if (tm && tm->tm_year == 2021 - 1900 && tm->tm_mon == 4 - 1
-      && tm->tm_mday == 28 && tm->tm_hour == 16 && tm->tm_min == 24
-      && 0 < tm->tm_isdst)
-    {
-      int has_leap_seconds = tm->tm_sec != now.tv_sec % 60;
-      p = "now - 35 years";
-      ASSERT (parse_datetime (&result, p, &now));
-      LOG (p, now, result);
-      /* Allow "now - 35 years" to mean either (a) the naive sense
-         where subtracting 35 years from 2021-04-28 16:24:50 yields
-         1986-04-28 16:24:50, or (b) the pedantic sense where it
-         yields 1986-04-28 15:24:50 (an hour earlier) due to adjusting
-         for the switch from DST in 2021 to standard time in 1986.  */
-      time_t naive = 515107490 + (has_leap_seconds ? 13 : 0);
-      time_t pedantic = naive - 60 * 60;
-      ASSERT (result.tv_sec == naive || result.tv_sec == pedantic);
-    }
-
   /* Check that some "next Monday", "last Wednesday", etc. are correct.  */
-  ASSERT (setenv ("TZ", "UTC0", 1) == 0);
-  tzset ();
+  setenv ("TZ", "UTC0", 1);
   for (i = 0; day_table[i]; i++)
     {
-      unsigned int thur2 = SOME_TIMEPOINT + 7 * 24 * 3600; /* 2nd thursday */
+      unsigned int thur2 = 7 * 24 * 3600; /* 2nd thursday */
       char tmp[32];
       sprintf (tmp, "NEXT %s", day_table[i]);
       now.tv_sec = thur2 + 4711;
@@ -449,16 +398,6 @@ main (_GL_UNUSED int argc, char **argv)
       ASSERT (result.tv_sec == thur2 + ((i + 3) % 7 - 7) * 24 * 3600);
     }
 
-/* On native Windows, localtime() fails for all time_t values < 0.  */
-#if !(defined _WIN32 && !defined __CYGWIN__)
-  p = "1970-12-31T23:59:59+00:00 - 1 year";  /* Bug#50115 */
-  now.tv_sec = -1;
-  now.tv_nsec = 0;
-  ASSERT (parse_datetime (&result, p, &now));
-  LOG (p, now, result);
-  ASSERT (result.tv_sec == now.tv_sec
-          && result.tv_nsec == now.tv_nsec);
-
   p = "THURSDAY UTC+00";  /* The epoch was on Thursday.  */
   now.tv_sec = 0;
   now.tv_nsec = 0;
@@ -474,7 +413,6 @@ main (_GL_UNUSED int argc, char **argv)
   LOG (p, now, result);
   ASSERT (result.tv_sec == 24 * 3600
           && result.tv_nsec == now.tv_nsec);
-#endif
 
   /* Exercise a sign-extension bug.  Before July 2012, an input
      starting with a high-bit-set byte would be treated like "0".  */
@@ -493,12 +431,8 @@ main (_GL_UNUSED int argc, char **argv)
   ASSERT (   parse_datetime (&result, "TZ=\"\"", &now));
   ASSERT (   parse_datetime (&result, "TZ=\"\" ", &now));
   ASSERT (   parse_datetime (&result, " TZ=\"\"", &now));
-  /* Exercise patterns which may be valid or invalid, depending on the
-     platform.  */
-#if !defined __NetBSD__
   ASSERT (   parse_datetime (&result, "TZ=\"\\\\\"", &now));
   ASSERT (   parse_datetime (&result, "TZ=\"\\\"\"", &now));
-#endif
 
   /* Outlandishly-long time zone abbreviations should not cause problems.  */
   {
@@ -524,5 +458,5 @@ main (_GL_UNUSED int argc, char **argv)
             && result.tv_nsec == 123456789);
   }
 
-  return test_exit_status;
+  return 0;
 }
