@@ -1,9 +1,9 @@
 /* Test the recvfrom() function.
-   Copyright (C) 2011-2025 Free Software Foundation, Inc.
+   Copyright (C) 2011-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -58,5 +58,5 @@ main (void)
     ASSERT (errno == EBADF);
   }
 
-  return test_exit_status;
+  return 0;
 }
