@@ -16,7 +16,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +29,7 @@
 
 #if defined(_WIN32) || !defined(HAVE_LIBSECCOMP)
 
-int main(void)
+int main()
 {
 	exit(77);
 }
@@ -64,9 +65,10 @@ static void client_log_func(int level, const char *str)
 
 #define MAX_BUF 1024
 
-static ssize_t push(gnutls_transport_ptr_t tr, const void *data, size_t len)
+static ssize_t
+push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 {
-	int fd = (long int)tr;
+	int fd = (long int) tr;
 
 	return send(fd, data, len, 0);
 }
@@ -100,7 +102,9 @@ static void client(int fd, const char *prio)
 	gnutls_handshake_set_timeout(session, get_timeout());
 
 	/* Use default priorities */
-	assert(gnutls_priority_set_direct(session, prio, NULL) >= 0);
+	assert(gnutls_priority_set_direct(session,
+					  prio,
+					  NULL) >= 0);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
@@ -111,7 +115,8 @@ static void client(int fd, const char *prio)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret < 0) {
 		fail("client: Handshake failed\n");
@@ -124,16 +129,17 @@ static void client(int fd, const char *prio)
 
 	if (debug)
 		success("client: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	do {
-		ret = gnutls_record_recv(session, buffer, sizeof(buffer) - 1);
+		ret = gnutls_record_recv(session, buffer, sizeof(buffer)-1);
 	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 
 	if (ret == 0) {
 		if (debug)
-			success("client: Peer has closed the TLS connection\n");
+			success
+			    ("client: Peer has closed the TLS connection\n");
 		goto end;
 	} else if (ret < 0) {
 		fail("client: Error: %s\n", gnutls_strerror(ret));
@@ -142,11 +148,10 @@ static void client(int fd, const char *prio)
 
 	ret = gnutls_bye(session, GNUTLS_SHUT_RDWR);
 	if (ret < 0) {
-		fail("server: error in closing session: %s\n",
-		     gnutls_strerror(ret));
+		fail("server: error in closing session: %s\n", gnutls_strerror(ret));
 	}
 
-end:
+      end:
 
 	close(fd);
 
@@ -157,13 +162,14 @@ end:
 	gnutls_global_deinit();
 }
 
+
 /* These are global */
 pid_t child;
 
 static void terminate(void)
 {
 	int status;
-	assert(child);
+
 	kill(child, SIGTERM);
 	wait(&status);
 	exit(1);
@@ -187,8 +193,9 @@ static void server(int fd, const char *prio)
 
 	gnutls_certificate_allocate_credentials(&xcred);
 
-	ret = gnutls_certificate_set_x509_key_mem(
-		xcred, &server_cert, &server_key, GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_key_mem(xcred,
+					    &server_cert, &server_key,
+					    GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
@@ -199,7 +206,9 @@ static void server(int fd, const char *prio)
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	assert(gnutls_priority_set_direct(session, prio, NULL) >= 0);
+	assert(gnutls_priority_set_direct(session,
+					  prio,
+					  NULL)>=0);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
@@ -208,7 +217,8 @@ static void server(int fd, const char *prio)
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 	if (ret < 0) {
 		close(fd);
 		gnutls_deinit(session);
@@ -221,15 +231,15 @@ static void server(int fd, const char *prio)
 
 	if (debug)
 		success("server: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	/* see the Getting peer's information example */
 	/* print_info(session); */
 
 	memset(buffer, 1, sizeof(buffer));
 	do {
-		ret = gnutls_record_send(session, buffer, sizeof(buffer) - 1);
+		ret = gnutls_record_send(session, buffer, sizeof(buffer)-1);
 	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 
 	if (ret < 0) {
@@ -242,8 +252,7 @@ static void server(int fd, const char *prio)
 
 	ret = gnutls_bye(session, GNUTLS_SHUT_RDWR);
 	if (ret < 0) {
-		fail("server: error in closing session: %s\n",
-		     gnutls_strerror(ret));
+		fail("server: error in closing session: %s\n", gnutls_strerror(ret));
 	}
 
 	close(fd);
@@ -257,7 +266,8 @@ static void server(int fd, const char *prio)
 		success("server: finished\n");
 }
 
-static void run(const char *name, const char *prio)
+static 
+void run(const char *name, const char *prio)
 {
 	int fd[2];
 	int ret;
@@ -298,4 +308,4 @@ void doit(void)
 {
 	run("dtls1.2", "NORMAL:-KX-ALL:+ECDHE-RSA:-VERS-ALL:+VERS-DTLS1.2");
 }
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
