@@ -16,7 +16,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +29,7 @@
 
 #if defined(_WIN32)
 
-int main(void)
+int main()
 {
 	exit(77);
 }
@@ -71,9 +72,10 @@ static void client_log_func(int level, const char *str)
 #define MAX_BUF 1024
 #define MSG "Hello TLS"
 
-static ssize_t push(gnutls_transport_ptr_t tr, const void *data, size_t len)
+static ssize_t
+push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 {
-	int fd = (long int)tr;
+	int fd = (long int) tr;
 
 	return send(fd, data, len, 0);
 }
@@ -101,9 +103,12 @@ static void client(int fd, int server_init, const char *prio)
 
 	/* Use default priorities */
 	snprintf(buffer, sizeof(buffer), "%s:+ECDHE-RSA", prio);
-	assert(gnutls_priority_set_direct(session, buffer, NULL) >= 0);
+	assert(gnutls_priority_set_direct(session,
+					  buffer,
+					  NULL) >= 0);
 
-	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, clientx509cred);
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE,
+				clientx509cred);
 
 	gnutls_transport_set_int(session, fd);
 	gnutls_transport_set_push_function(session, push);
@@ -112,7 +117,8 @@ static void client(int fd, int server_init, const char *prio)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret < 0) {
 		fail("client: Handshake failed\n");
@@ -125,8 +131,8 @@ static void client(int fd, int server_init, const char *prio)
 
 	if (debug)
 		success("client: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	if (!server_init) {
 		sec_sleep(60);
@@ -134,30 +140,35 @@ static void client(int fd, int server_init, const char *prio)
 			success("Initiating client rehandshake\n");
 		do {
 			ret = gnutls_handshake(session);
-		} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+		}
+		while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 		if (ret < 0) {
 			fail("2nd client gnutls_handshake: %s\n",
 			     gnutls_strerror(ret));
-			exit(1);
+			terminate();
 		}
 	} else {
 		do {
 			ret = gnutls_record_recv(session, buffer, MAX_BUF);
-		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+		} while (ret == GNUTLS_E_AGAIN
+			 || ret == GNUTLS_E_INTERRUPTED);
 	}
 
 	if (ret == 0) {
 		if (debug)
-			success("client: Peer has closed the TLS connection\n");
+			success
+			    ("client: Peer has closed the TLS connection\n");
 		goto end;
 	} else if (ret < 0) {
 		if (server_init && ret == GNUTLS_E_REHANDSHAKE) {
 			if (debug)
-				success("Initiating rehandshake due to server request\n");
+				success
+				    ("Initiating rehandshake due to server request\n");
 			do {
 				ret = gnutls_handshake(session);
-			} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+			}
+			while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 		}
 
 		if (ret != 0) {
@@ -171,7 +182,7 @@ static void client(int fd, int server_init, const char *prio)
 	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 	gnutls_bye(session, GNUTLS_SHUT_WR);
 
-end:
+      end:
 
 	close(fd);
 
@@ -182,13 +193,15 @@ end:
 	gnutls_global_deinit();
 }
 
+
 /* These are global */
 pid_t child;
 
+
 static void terminate(void)
 {
 	int status;
-	assert(child);
+
 	kill(child, SIGTERM);
 	wait(&status);
 	exit(1);
@@ -210,9 +223,9 @@ static void server(int fd, int server_init, const char *prio)
 	}
 
 	assert(gnutls_certificate_allocate_credentials(&serverx509cred) >= 0);
-	assert(gnutls_certificate_set_x509_key_mem(serverx509cred, &server_cert,
-						   &server_key,
-						   GNUTLS_X509_FMT_PEM) >= 0);
+	assert(gnutls_certificate_set_x509_key_mem(serverx509cred,
+					    &server_cert, &server_key,
+					    GNUTLS_X509_FMT_PEM) >= 0);
 
 	gnutls_init(&session, GNUTLS_SERVER | GNUTLS_DATAGRAM);
 	gnutls_dtls_set_mtu(session, 1500);
@@ -221,16 +234,20 @@ static void server(int fd, int server_init, const char *prio)
 	 * are adequate.
 	 */
 	snprintf(buffer, sizeof(buffer), "%s:+ECDHE-RSA", prio);
-	assert(gnutls_priority_set_direct(session, buffer, NULL) >= 0);
+	assert(gnutls_priority_set_direct(session,
+					  buffer,
+					  NULL) >= 0);
 
-	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, serverx509cred);
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE,
+				serverx509cred);
 
 	gnutls_transport_set_int(session, fd);
 	gnutls_transport_set_push_function(session, push);
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 	if (ret < 0) {
 		close(fd);
 		gnutls_deinit(session);
@@ -243,8 +260,8 @@ static void server(int fd, int server_init, const char *prio)
 
 	if (debug)
 		success("server: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	/* see the Getting peer's information example */
 	/* print_info(session); */
@@ -254,7 +271,8 @@ static void server(int fd, int server_init, const char *prio)
 			success("server: Sending dummy packet\n");
 		ret = gnutls_rehandshake(session);
 		if (ret < 0) {
-			fail("gnutls_rehandshake: %s\n", gnutls_strerror(ret));
+			fail("gnutls_rehandshake: %s\n",
+			     gnutls_strerror(ret));
 			terminate();
 		}
 
@@ -262,7 +280,8 @@ static void server(int fd, int server_init, const char *prio)
 			success("server: Initiating rehandshake\n");
 		do {
 			ret = gnutls_handshake(session);
-		} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+		}
+		while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 		if (ret < 0) {
 			fail("server: 2nd gnutls_handshake: %s\n",
@@ -276,38 +295,43 @@ static void server(int fd, int server_init, const char *prio)
 
 		do {
 			ret = gnutls_record_recv(session, buffer, MAX_BUF);
-		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+		} while (ret == GNUTLS_E_AGAIN
+			 || ret == GNUTLS_E_INTERRUPTED);
 
 		if (ret == 0) {
 			if (debug)
-				success("server: Peer has closed the GnuTLS connection\n");
+				success
+				    ("server: Peer has closed the GnuTLS connection\n");
 			break;
 		} else if (ret < 0) {
 			if (!server_init && ret == GNUTLS_E_REHANDSHAKE) {
 				if (debug)
-					success("Initiating rehandshake due to client request\n");
+					success
+					    ("Initiating rehandshake due to client request\n");
 				do {
 					ret = gnutls_handshake(session);
-				} while (ret < 0 &&
-					 gnutls_error_is_fatal(ret) == 0);
+				}
+				while (ret < 0
+					&& gnutls_error_is_fatal(ret) == 0);
 				if (ret == 0)
 					break;
 			}
 
-			fail("server: Received corrupted data(%s). Closing...\n",
-			     gnutls_strerror(ret));
+			fail("server: Received corrupted data(%s). Closing...\n", gnutls_strerror(ret));
 			terminate();
 		} else if (ret > 0) {
 			/* echo data back to the client
 			 */
 			do {
-				ret = gnutls_record_send(session, buffer,
-							 strlen(buffer));
-			} while (ret == GNUTLS_E_AGAIN ||
-				 ret == GNUTLS_E_INTERRUPTED);
+				ret =
+				    gnutls_record_send(session, buffer,
+							strlen(buffer));
+			} while (ret == GNUTLS_E_AGAIN
+				 || ret == GNUTLS_E_INTERRUPTED);
 		}
 	}
 
+
 	/* do not wait for the peer to close the connection.
 	 */
 	gnutls_bye(session, GNUTLS_SHUT_WR);
@@ -357,10 +381,8 @@ static void start(int server_initiated, const char *prio)
 
 void doit(void)
 {
-	start(0,
-	      "NONE:+VERS-DTLS1.2:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+CURVE-ALL");
-	start(1,
-	      "NONE:+VERS-DTLS1.2:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+CURVE-ALL");
+	start(0, "NONE:+VERS-DTLS1.2:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+CURVE-ALL");
+	start(1, "NONE:+VERS-DTLS1.2:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+CURVE-ALL");
 }
 
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
