@@ -17,11 +17,12 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -41,7 +42,7 @@ static void tls_log_func(int level, const char *str)
 	fprintf(stderr, "%s|<%d>| %s", side, level, str);
 }
 
-static time_t mytime(time_t *t)
+static time_t mytime(time_t * t)
 {
 	time_t then = 1490171562;
 
@@ -79,18 +80,18 @@ void doit(void)
 	gnutls_global_set_time_function(mytime);
 
 	/* Init server */
-	assert(gnutls_certificate_allocate_credentials(&serverx509cred) >= 0);
+	assert(gnutls_certificate_allocate_credentials(&serverx509cred)>=0);
 	ret = gnutls_certificate_set_x509_key_mem(serverx509cred,
-						  &server_ca3_ipaddr_cert,
-						  &server_ca3_key,
-						  GNUTLS_X509_FMT_PEM);
+					    &server_ca3_ipaddr_cert, &server_ca3_key,
+					    GNUTLS_X509_FMT_PEM);
 	if (ret < 0) {
 		fail("could not import cert: %s\n", gnutls_strerror(ret));
 	}
 
-	assert(gnutls_init(&server, GNUTLS_SERVER) >= 0);
-	gnutls_credentials_set(server, GNUTLS_CRD_CERTIFICATE, serverx509cred);
-	assert(gnutls_set_default_priority(server) >= 0);
+	assert(gnutls_init(&server, GNUTLS_SERVER)>=0);
+	gnutls_credentials_set(server, GNUTLS_CRD_CERTIFICATE,
+				serverx509cred);
+	assert(gnutls_set_default_priority(server)>=0);
 	gnutls_transport_set_push_function(server, server_push);
 	gnutls_transport_set_pull_function(server, server_pull);
 	gnutls_transport_set_ptr(server, server);
@@ -100,8 +101,7 @@ void doit(void)
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_certificate_set_x509_trust_mem(clientx509cred, &ca3_cert,
-						    GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_trust_mem(clientx509cred, &ca3_cert, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
@@ -110,7 +110,7 @@ void doit(void)
 		exit(1);
 
 	ret = gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE,
-				     clientx509cred);
+				clientx509cred);
 	if (ret < 0)
 		exit(1);
 
@@ -123,19 +123,19 @@ void doit(void)
 
 	/* attempt to verify */
 	{
+
 		/* try hostname - which is invalid */
 		memset(data, 0, sizeof(data));
 
 		data[0].type = GNUTLS_DT_DNS_HOSTNAME;
-		data[0].data = (void *)"localhost1";
+		data[0].data = (void*)"localhost1";
 
 		data[1].type = GNUTLS_DT_KEY_PURPOSE_OID;
-		data[1].data = (void *)GNUTLS_KP_TLS_WWW_SERVER;
+		data[1].data = (void*)GNUTLS_KP_TLS_WWW_SERVER;
 
 		ret = gnutls_certificate_verify_peers(client, data, 2, &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
@@ -148,16 +148,15 @@ void doit(void)
 		memset(data, 0, sizeof(data));
 
 		data[0].type = GNUTLS_DT_IP_ADDRESS;
-		data[0].data = (void *)"\x01\x00\x01\x02";
+		data[0].data = (void*)"\x01\x00\x01\x02";
 		data[0].size = 4;
 
 		data[1].type = GNUTLS_DT_KEY_PURPOSE_OID;
-		data[1].data = (void *)GNUTLS_KP_TLS_WWW_SERVER;
+		data[1].data = (void*)GNUTLS_KP_TLS_WWW_SERVER;
 
 		ret = gnutls_certificate_verify_peers(client, data, 2, &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
@@ -170,52 +169,44 @@ void doit(void)
 		memset(data, 0, sizeof(data));
 
 		data[0].type = GNUTLS_DT_IP_ADDRESS;
-		data[0].data = (void *)"\x7f\x00\x00\x01";
+		data[0].data = (void*)"\x7f\x00\x00\x01";
 		data[0].size = 4;
 
 		data[1].type = GNUTLS_DT_KEY_PURPOSE_OID;
-		data[1].data = (void *)GNUTLS_KP_TLS_WWW_SERVER;
+		data[1].data = (void*)GNUTLS_KP_TLS_WWW_SERVER;
 
 		ret = gnutls_certificate_verify_peers(client, data, 2, &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
 		if (status != 0) {
-			assert(gnutls_certificate_verification_status_print(
-				       status, GNUTLS_CRT_X509, &t, 0) >= 0);
+			assert(gnutls_certificate_verification_status_print(status, GNUTLS_CRT_X509, &t, 0)>=0);
 			fail("could not verify: %s/%.4x!\n", t.data, status);
 		}
 
 		/* try the other verification functions */
-		ret = gnutls_certificate_verify_peers3(client, "127.0.0.1",
-						       &status);
+		ret = gnutls_certificate_verify_peers3(client, "127.0.0.1", &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
 		if (status != 0) {
-			assert(gnutls_certificate_verification_status_print(
-				       status, GNUTLS_CRT_X509, &t, 0) >= 0);
+			assert(gnutls_certificate_verification_status_print(status, GNUTLS_CRT_X509, &t, 0)>=0);
 			fail("could not verify: %s/%.4x!\n", t.data, status);
 		}
 	}
 
 	{
 		/* change the flags */
-		gnutls_certificate_set_verify_flags(
-			clientx509cred, GNUTLS_VERIFY_DO_NOT_ALLOW_IP_MATCHES);
+		gnutls_certificate_set_verify_flags(clientx509cred, GNUTLS_VERIFY_DO_NOT_ALLOW_IP_MATCHES);
 
 		/* now the compatibility option should fail */
-		ret = gnutls_certificate_verify_peers3(client, "127.0.0.1",
-						       &status);
+		ret = gnutls_certificate_verify_peers3(client, "127.0.0.1", &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
@@ -227,15 +218,14 @@ void doit(void)
 		memset(data, 0, sizeof(data));
 
 		data[0].type = GNUTLS_DT_DNS_HOSTNAME;
-		data[0].data = (void *)"127.0.0.1";
+		data[0].data = (void*)"127.0.0.1";
 
 		data[1].type = GNUTLS_DT_KEY_PURPOSE_OID;
-		data[1].data = (void *)GNUTLS_KP_TLS_WWW_SERVER;
+		data[1].data = (void*)GNUTLS_KP_TLS_WWW_SERVER;
 
 		ret = gnutls_certificate_verify_peers(client, data, 2, &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
@@ -248,22 +238,20 @@ void doit(void)
 		memset(data, 0, sizeof(data));
 
 		data[0].type = GNUTLS_DT_IP_ADDRESS;
-		data[0].data = (void *)"\x7f\x00\x00\x01";
+		data[0].data = (void*)"\x7f\x00\x00\x01";
 		data[0].size = 4;
 
 		data[1].type = GNUTLS_DT_KEY_PURPOSE_OID;
-		data[1].data = (void *)GNUTLS_KP_TLS_WWW_SERVER;
+		data[1].data = (void*)GNUTLS_KP_TLS_WWW_SERVER;
 
 		ret = gnutls_certificate_verify_peers(client, data, 2, &status);
 		if (ret < 0) {
-			fail("could not verify certificate: %s\n",
-			     gnutls_strerror(ret));
+			fail("could not verify certificate: %s\n", gnutls_strerror(ret));
 			exit(1);
 		}
 
 		if (status != 0) {
-			assert(gnutls_certificate_verification_status_print(
-				       status, GNUTLS_CRT_X509, &t, 0) >= 0);
+			assert(gnutls_certificate_verification_status_print(status, GNUTLS_CRT_X509, &t, 0)>=0);
 			fail("could not verify: %s/%.4x!\n", t.data, status);
 		}
 	}
