@@ -64,39 +64,36 @@ static void sign_verify_data(gnutls_pk_algorithm_t algorithm, gnutls_x509_privke
 	gnutls_pubkey_t pubkey;
 	gnutls_datum_t signature;
 	gnutls_digest_algorithm_t digest;
+	unsigned vflags = 0;
 
-	if (algorithm == GNUTLS_PK_EDDSA_ED25519)
-		digest = GNUTLS_DIG_SHA512;
-	else if (algorithm == GNUTLS_PK_GOST_01)
-		digest = GNUTLS_DIG_GOSTR_94;
-	else if (algorithm == GNUTLS_PK_GOST_12_256)
-		digest = GNUTLS_DIG_STREEBOG_256;
-	else if (algorithm == GNUTLS_PK_GOST_12_512)
-		digest = GNUTLS_DIG_STREEBOG_512;
-	else
-		digest = GNUTLS_DIG_SHA256;
-
-	/* sign arbitrary data */
 	assert(gnutls_privkey_init(&privkey) >= 0);
 
 	ret = gnutls_privkey_import_x509(privkey, pkey, 0);
 	if (ret < 0)
 		fail("gnutls_privkey_import_x509\n");
 
-	ret = gnutls_privkey_sign_data(privkey, digest, 0,
-					&raw_data, &signature);
-	if (ret < 0)
-		fail("gnutls_x509_privkey_sign_data\n");
-
-	/* verify data */
 	assert(gnutls_pubkey_init(&pubkey) >= 0);
 
 	ret = gnutls_pubkey_import_privkey(pubkey, privkey, 0, 0);
 	if (ret < 0)
 		fail("gnutls_pubkey_import_privkey\n");
 
+	ret = gnutls_pubkey_get_preferred_hash_algorithm (pubkey, &digest, NULL);
+	if (ret < 0)
+		fail("gnutls_pubkey_get_preferred_hash_algorithm\n");
+
+	if (digest == GNUTLS_DIG_GOSTR_94)
+		vflags |= GNUTLS_VERIFY_ALLOW_BROKEN;
+
+	/* sign arbitrary data */
+	ret = gnutls_privkey_sign_data(privkey, digest, 0,
+					&raw_data, &signature);
+	if (ret < 0)
+		fail("gnutls_privkey_sign_data\n");
+
+	/* verify data */
 	ret = gnutls_pubkey_verify_data2(pubkey, gnutls_pk_to_sign(gnutls_pubkey_get_pk_algorithm(pubkey, NULL),digest),
-				0, &raw_data, &signature);
+				         vflags, &raw_data, &signature);
 	if (ret < 0)
 		fail("gnutls_pubkey_verify_data2\n");
 
@@ -122,15 +119,24 @@ void doit(void)
 		for (algorithm = GNUTLS_PK_RSA; algorithm <= GNUTLS_PK_MAX;
 		     algorithm++) {
 			if (algorithm == GNUTLS_PK_DH ||
-			    algorithm == GNUTLS_PK_ECDH_X25519)
+			    algorithm == GNUTLS_PK_ECDH_X25519 ||
+			    algorithm == GNUTLS_PK_ECDH_X448)
 				continue;
 
-			if (gnutls_fips140_mode_enabled() &&
-			    (algorithm == GNUTLS_PK_GOST_01 ||
-			     algorithm == GNUTLS_PK_GOST_12_256 ||
-			     algorithm == GNUTLS_PK_GOST_12_512))
+			if (algorithm == GNUTLS_PK_GOST_01 ||
+			    algorithm == GNUTLS_PK_GOST_12_256 ||
+			    algorithm == GNUTLS_PK_GOST_12_512) {
+				/* Skip GOST algorithms:
+				 * - If they are disabled by ./configure option
+				 * - Or in FIPS140 mode
+				 */
+#ifdef ENABLE_GOST
+				if (gnutls_fips140_mode_enabled())
+					continue;
+#else
 				continue;
-
+#endif
+			}
 
 			ret = gnutls_x509_privkey_init(&pkey);
 			if (ret < 0) {
