@@ -18,7 +18,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  */
 
 /* Parts copied from GnuTLS example programs. */
@@ -73,6 +73,7 @@ struct params_res {
 	int enable_session_ticket_server;
 	int enable_session_ticket_client;
 	int expect_resume;
+	int call_post_client_hello;
 	int client_cert;
 	int first_no_ext_master;
 	int second_no_ext_master;
@@ -95,6 +96,12 @@ struct params_res resume_tests[] = {
 	 .enable_session_ticket_server = 0,
 	 .enable_session_ticket_client = 0,
 	 .expect_resume = 1},
+	{.desc = "try to resume from db with post_client_hello",
+	 .enable_db = 1,
+	 .enable_session_ticket_server = 0,
+	 .enable_session_ticket_client = 0,
+	 .call_post_client_hello = 1,
+	 .expect_resume = 1},
 	{.desc = "try to resume from db using resumed session's data",
 	 .enable_db = 1,
 	 .enable_session_ticket_server = 0,
@@ -131,6 +138,12 @@ struct params_res resume_tests[] = {
 	 .enable_session_ticket_client = 1,
 	 .change_ciphersuite = 1,
 	 .expect_resume = 1},
+	{.desc = "try to resume from session ticket with post_client_hello",
+	 .enable_db = 0,
+	 .enable_session_ticket_server = 1,
+	 .enable_session_ticket_client = 1,
+	 .call_post_client_hello = 1,
+	 .expect_resume = 1},
 #endif
 #if defined(TLS13) && !defined(USE_PSK)
 	{.desc = "try to resume from session ticket (early start)",
@@ -241,6 +254,13 @@ static void tls_log_func(int level, const char *str)
 		str);
 }
 
+static int post_client_hello_callback(gnutls_session_t session)
+{
+	/* switches the supported ciphersuites to something compatible */
+	assert(gnutls_priority_set_direct(session, gnutls_session_get_ptr(session), NULL) >= 0);
+	return 0;
+}
+
 static int hsk_hook_cb(gnutls_session_t session, unsigned int htype, unsigned post,
 			unsigned int incoming, const gnutls_datum_t *_msg)
 {
@@ -321,8 +341,6 @@ static void verify_alpn(gnutls_session_t session, struct params_res *params, uns
 
 static void verify_group(gnutls_session_t session, gnutls_group_t *group, unsigned counter)
 {
-	int ret;
-
 	if (counter == 0) {
 		*group = gnutls_group_get(session);
 		return;
@@ -371,14 +389,17 @@ static void verify_server_params(gnutls_session_t session, unsigned counter, str
 #if defined(USE_X509)
 	unsigned int l;
 
+	if (gnutls_certificate_type_get(session) != GNUTLS_CRT_X509)
+		fail("did not find the expected X509 certificate type! (%d)\n", gnutls_certificate_type_get(session));
+
 	if (counter == 0 && gnutls_certificate_get_ours(session) == NULL)
-		fail("no certificate returned on server side (%s)\n", counter?"resumed session":"first session");
+		fail("no certificate returned on server side (%s)\n", counter ? "resumed session" : "first session");
 	else if (counter != 0 && gnutls_certificate_get_ours(session) != NULL)
-		fail("certificate was returned on server side (%s)\n", counter?"resumed session":"first session");
+		fail("certificate was returned on server side (%s)\n", counter ? "resumed session" : "first session");
 
 	if (params->client_cert) {
 		if (gnutls_certificate_get_peers(session, &l) == NULL || l < 1)
-			fail("no client certificate returned on server side (%s)\n", counter?"resumed session":"first session");
+			fail("no client certificate returned on server side (%s)\n", counter ? "resumed session" : "first session");
 	}
 #endif
 
@@ -403,7 +424,6 @@ static void verify_server_params(gnutls_session_t session, unsigned counter, str
 		}
 	}
 
- finish:
 	return;
 }
 
@@ -412,7 +432,7 @@ static void verify_client_params(gnutls_session_t session, unsigned counter)
 #if defined(USE_X509)
 	unsigned int l;
 	if (gnutls_certificate_get_peers(session, &l) == NULL || l < 1)
-		fail("no server certificate returned on client side (%s)\n", counter?"resumed session":"first session");
+		fail("no server certificate returned on client side (%s)\n", counter ? "resumed session" : "first session");
 #else
 	return;
 #endif
@@ -540,7 +560,7 @@ static void client(int sds[], struct params_res *params)
 
 		/* Perform the TLS handshake
 		 */
-		gnutls_handshake_set_timeout(session, 20 * 1000);
+		gnutls_handshake_set_timeout(session, get_timeout());
 		do {
 			ret = gnutls_handshake(session);
 		} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
@@ -714,7 +734,7 @@ pskfunc(gnutls_session_t session, const char *username,
 
 static void server(int sds[], struct params_res *params)
 {
-	size_t t;
+	int t;
 	int ret;
 	gnutls_session_t session;
 	char buffer[MAX_BUF + 1];
@@ -807,7 +827,14 @@ static void server(int sds[], struct params_res *params)
 		gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, serverx509cred);
 #endif
 		gnutls_transport_set_int(session, sd);
-		gnutls_handshake_set_timeout(session, 20 * 1000);
+		gnutls_handshake_set_timeout(session, get_timeout());
+
+		if (params->call_post_client_hello) {
+			gnutls_session_set_ptr(session, PRIO_STR);
+			gnutls_handshake_set_post_client_hello_function(session,
+									post_client_hello_callback);
+		}
+
 
 		do {
 			ret = gnutls_handshake(session);
@@ -873,7 +900,6 @@ static void server(int sds[], struct params_res *params)
 	}
 
 	gnutls_free(session_ticket_key.data);
-	session_ticket_key.data = NULL;
 
 	if (debug)
 		success("server: finished\n");
@@ -968,7 +994,7 @@ static void wrap_db_deinit(void)
 static int
 wrap_db_store(void *dbf, gnutls_datum_t key, gnutls_datum_t data)
 {
-	time_t t, now = time(0);
+	time_t t, e, now = time(0);
 
 #ifdef DEBUG_CACHE
 	if (debug) {
@@ -994,6 +1020,13 @@ wrap_db_store(void *dbf, gnutls_datum_t key, gnutls_datum_t data)
 		exit(1);
 	}
 
+	/* check the correctness of gnutls_db_check_entry_expire_time() */
+	e = gnutls_db_check_entry_expire_time(&data);
+	if (e < t) {
+		fail("Time returned by gnutls_db_check_entry_expire_time is bogus\n");
+		exit(1);
+	}
+
 	if (cache_db == NULL)
 		return -1;
 
