@@ -45,13 +45,6 @@ SERV="${SERV} -q"
 
 . "${srcdir}/scripts/common.sh"
 
-: ${ac_cv_sizeof_time_t=8}
-if test "${ac_cv_sizeof_time_t}" -ge 8; then
-	ATTIME_VALID="2038-10-12"  # almost the pregenerated cert expiration
-else
-	ATTIME_VALID="2037-12-31"  # before 2038
-fi
-
 KEY1=${srcdir}/../doc/credentials/x509/example.com-key.pem
 CERT1=${srcdir}/../doc/credentials/x509/example.com-cert.pem
 CA1=${srcdir}/../doc/credentials/x509/ca.pem
@@ -66,22 +59,22 @@ launch_server --echo --sni-hostname-fatal --sni-hostname example.com --x509keyfi
 PID=$!
 wait_server ${PID}
 
-${VALGRIND} "${CLI}" --attime "${ATTIME_VALID}" -p "${PORT}" 127.0.0.1 ${OPTS} --priority "NORMAL:-VERS-ALL:+VERS-TLS1.2" --x509cafile ${CA1} </dev/null >/dev/null || \
+${VALGRIND} "${CLI}" -p "${PORT}" 127.0.0.1 ${OPTS} --priority "NORMAL:-VERS-ALL:+VERS-TLS1.2" --x509cafile ${CA1} </dev/null >/dev/null || \
 	fail ${PID} "1. handshake should have succeeded!"
 
-${VALGRIND} "${CLI}" --attime "${ATTIME_VALID}" -p "${PORT}" 127.0.0.1 ${NOOPTS} --priority "NORMAL:-VERS-ALL:+VERS-TLS1.2" --x509cafile ${CA1} </dev/null >/dev/null && \
+${VALGRIND} "${CLI}" -p "${PORT}" 127.0.0.1 ${NOOPTS} --priority "NORMAL:-VERS-ALL:+VERS-TLS1.2" --x509cafile ${CA1} </dev/null >/dev/null && \
 	fail ${PID} "2. handshake should have failed!"
 
-${VALGRIND} "${CLI}" --attime "${ATTIME_VALID}" -p "${PORT}" 127.0.0.1 ${OPTS} --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null || \
+${VALGRIND} "${CLI}" -p "${PORT}" 127.0.0.1 ${OPTS} --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null || \
 	fail ${PID} "3. handshake should have succeeded!"
 
-${VALGRIND} "${CLI}" --attime "${ATTIME_VALID}" -p "${PORT}" 127.0.0.1 ${NOOPTS} --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null && \
+${VALGRIND} "${CLI}" -p "${PORT}" 127.0.0.1 ${NOOPTS} --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null && \
 	fail ${PID} "4. handshake should have failed!"
 
-${VALGRIND} "${CLI}" --attime "${ATTIME_VALID}" -p "${PORT}" 127.0.0.1 --sni-hostname example.com --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null && \
+${VALGRIND} "${CLI}" -p "${PORT}" 127.0.0.1 --sni-hostname example.com --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null && \
 	fail ${PID} "5. handshake should have failed!"
 
-${VALGRIND} "${CLI}" --attime "${ATTIME_VALID}" -p "${PORT}" 127.0.0.1 --sni-hostname example.com. --verify-hostname example.com. --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null || \
+${VALGRIND} "${CLI}" -p "${PORT}" 127.0.0.1 --sni-hostname example.com. --verify-hostname example.com. --priority "NORMAL" --x509cafile ${CA1} </dev/null >/dev/null || \
 	fail ${PID} "6. handshake should have succeeded!"
 
 kill ${PID}
