@@ -18,12 +18,12 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>
 #
 
-srcdir="${srcdir:-.}"
-builddir="${builddir:-.}"
+: ${srcdir=.}
+: ${builddir=.}
 
 . "${srcdir}/scripts/common.sh"
 
-check_for_datefudge
+skip_if_no_datefudge
 
 datefudge -s 2019-04-12 "${builddir}/tls13/prf-early" "$@"
 exit $?
