@@ -16,7 +16,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +33,7 @@
 
 #if defined(_WIN32)
 
-int main(void)
+int main()
 {
 	exit(77);
 }
@@ -72,7 +73,7 @@ static void client_log_func(int level, const char *str)
 #define HANDSHAKE_ID_POS (38)
 static ssize_t odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 {
-	uint8_t *d = (void *)data;
+	uint8_t *d = (void*)data;
 	int fd = (long)tr;
 	uint16_t csize, osize;
 	int pos;
@@ -81,35 +82,34 @@ static ssize_t odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 		uint8_t isize;
 
 		/* skip session ID (this can be non-empty in TLS 1.3) */
-		isize = d[RECORD_PAYLOAD_POS + HANDSHAKE_ID_POS];
+		isize = d[RECORD_PAYLOAD_POS+HANDSHAKE_ID_POS];
 		isize += 1;
 
 		/* skip ciphersuites */
-		csize = d[RECORD_PAYLOAD_POS + HANDSHAKE_ID_POS + isize + 1] +
-			(d[RECORD_PAYLOAD_POS + HANDSHAKE_ID_POS + isize] << 8);
+		csize = d[RECORD_PAYLOAD_POS+HANDSHAKE_ID_POS+isize+1] + (d[RECORD_PAYLOAD_POS+HANDSHAKE_ID_POS+isize] << 8);
 		csize += 2;
 
 		/* skip compression methods */
-		osize = d[RECORD_PAYLOAD_POS + HANDSHAKE_ID_POS + isize + csize];
+		osize = d[RECORD_PAYLOAD_POS+HANDSHAKE_ID_POS+isize+csize];
 		osize += 1;
 
-		pos = RECORD_PAYLOAD_POS + HANDSHAKE_ID_POS + isize + csize +
-		      osize;
+		pos = RECORD_PAYLOAD_POS+HANDSHAKE_ID_POS+isize+csize+osize;
 
 		if (reduce) {
-			if (d[pos + 1] != 0x00) {
-				d[pos + 1] = d[pos + 1] - 1;
+			if (d[pos+1] != 0x00) {
+				d[pos+1] = d[pos+1] - 1;
 			} else {
 				d[pos] = d[pos] - 1;
-				d[pos + 1] = 0xff;
+				d[pos+1] = 0xff;
 			}
 		} else {
-			if (d[pos + 1] != 0xff) {
-				d[pos + 1] = d[pos + 1] + 1;
+			if (d[pos+1] != 0xff) {
+				d[pos+1] = d[pos+1] + 1;
 			} else {
 				d[pos] = d[pos] + 1;
-				d[pos + 1] = 0x00;
+				d[pos+1] = 0x00;
 			}
+		
 		}
 	}
 
@@ -150,7 +150,8 @@ static void client(int fd, const char *prio)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret >= 0) {
 		fail("client: Handshake succeeded!\n");
@@ -166,13 +167,14 @@ static void client(int fd, const char *prio)
 	gnutls_global_deinit();
 }
 
+
 /* These are global */
 pid_t child;
 
 static void terminate(void)
 {
 	int status;
-	assert(child);
+
 	kill(child, SIGTERM);
 	wait(&status);
 	exit(1);
@@ -195,8 +197,9 @@ static void server(int fd, const char *prio)
 
 	gnutls_certificate_allocate_credentials(&xcred);
 
-	ret = gnutls_certificate_set_x509_key_mem(
-		xcred, &server_cert, &server_key, GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_key_mem(xcred,
+					    &server_cert, &server_key,
+					    GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
@@ -214,7 +217,8 @@ static void server(int fd, const char *prio)
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret != GNUTLS_E_UNEXPECTED_EXTENSIONS_LENGTH) {
 		close(fd);
@@ -286,4 +290,4 @@ void doit(void)
 	start("NORMAL:-VERS-ALL:+VERS-TLS1.3");
 	start("NORMAL");
 }
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
