@@ -20,9 +20,11 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>
  */
 
-#ifndef UTILS_H
-#define UTILS_H
+#ifndef GNUTLS_TESTS_UTILS_H
+#define GNUTLS_TESTS_UTILS_H
 
+#include <assert.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -41,13 +43,16 @@
 # error tests cannot be compiled with NDEBUG defined
 #endif
 
-#if _GNUTLS_GCC_VERSION >= 70100
-#define FALLTHROUGH      __attribute__ ((fallthrough))
-#endif
-
 #ifndef FALLTHROUGH
+#if _GNUTLS_GCC_VERSION >= 70100
+# define FALLTHROUGH      __attribute__ ((fallthrough))
+#else
 # define FALLTHROUGH
 #endif
+#endif
+
+/* number of elements within an array */
+#define countof(a) (sizeof(a)/sizeof(*(a)))
 
 inline static int global_init(void)
 {
@@ -174,4 +179,32 @@ inline static void check_wait_status_for_sig(int status)
 	_check_wait_status(status, 1);
 }
 
-#endif				/* UTILS_H */
+inline static unsigned int get_timeout(void) {
+	const char *envvar;
+	unsigned long int ul;
+
+	envvar = getenv("GNUTLS_TEST_TIMEOUT");
+	if (!envvar || *envvar == '\0')
+		return 20 * 1000;
+
+	ul = strtoul(envvar, NULL, 10);
+	assert(ul <= UINT_MAX);
+
+	return (unsigned int) ul;
+}
+
+inline static unsigned int get_dtls_retransmit_timeout(void) {
+	const char *envvar;
+	unsigned long int ul;
+
+	envvar = getenv("GNUTLS_TEST_DTLS_RETRANSMIT_TIMEOUT");
+	if (!envvar || *envvar == '\0')
+		return get_timeout() / 10;
+
+	ul = strtoul(envvar, NULL, 10);
+	assert(ul <= UINT_MAX);
+
+	return (unsigned int) ul;
+}
+
+#endif /* GNUTLS_TESTS_UTILS_H */
